=== modified file 'Makefile'
--- Makefile	2007-10-28 17:59:38 +0000
+++ Makefile	2008-08-08 02:33:41 +0000
@@ -1,7 +1,58 @@
-CXXFLAGS=-Wall -W -g
-LDFLAGS=-lgnutls
-
-all: client server
+WARN=-O -Wall -Wformat=2 -Winit-self -Wmissing-include-dirs -Wswitch-default -Wswitch-enum -Wunused-parameter -Wstrict-aliasing=2 -Wextra -Wfloat-equal -Wundef -Wshadow -Wunsafe-loop-optimizations -Wpointer-arith -Wbad-function-cast -Wcast-qual -Wcast-align -Wwrite-strings -Wconversion -Wstrict-prototypes -Wold-style-definition -Wpacked -Wnested-externs -Wunreachable-code -Winline -Wvolatile-register-var 
+DEBUG=-ggdb3
+# For info about _FORTIFY_SOURCE, see
+# <http://gcc.gnu.org/ml/gcc-patches/2004-09/msg02055.html>
+FORTIFY=-D_FORTIFY_SOURCE=2 # -fstack-protector-all
+#COVERAGE=--coverage
+OPTIMIZE=-Os
+LANGUAGE=-std=gnu99
+
+# Do not change these two
+CFLAGS=$(WARN) $(DEBUG) $(FORTIFY) $(COVERAGE) $(OPTIMIZE) $(LANGUAGE)
+LDFLAGS=$(COVERAGE)
+
+DOCBOOKTOMAN=xsltproc --nonet --param man.charmap.use.subset "0" --param make.year.ranges "1" --param make.single.year.ranges "1" --param man.output.quietly "1"
+
+PROGS=mandos-client plugins.d/password-request plugins.d/password-prompt
+DOCS=mandos.8 mandos-client.8mandos plugins.d/password-request.8mandos plugins.d/password-prompt.8mandos mandos.conf.5 mandos-clients.conf.5
+
+objects=$(shell for p in $(PROGS); do echo $${p}.o; done)
+
+all: $(PROGS) $(DOCS)
+
+%.5: %.xml
+	$(DOCBOOKTOMAN) $^
+
+%.8: %.xml
+	$(DOCBOOKTOMAN) $^
+
+%.8mandos: %.xml
+	$(DOCBOOKTOMAN) $^
+	-mv ./$(shell basename $@) $@ 2>/dev/null
+
+mandos-client: mandos-client.o
+	$(LINK.o) -lgnutls $(COMMON) $^ $(LOADLIBES) $(LDLIBS) -o $@
+
+plugins.d/password-request: plugins.d/password-request.o
+	$(LINK.o) -lgnutls -lavahi-core -lgpgme $(COMMON) $^ $(LOADLIBES) $(LDLIBS) -o $@
+
+plugins.d/password-prompt: plugins.d/password-prompt.o
+	$(LINK.o) $(COMMON) $^ $(LOADLIBES) $(LDLIBS) -o $@
+
+.PHONY : all clean distclean run-client run-server
 
 clean:
-	rm -f server client
+	-rm --force $(PROGS) $(objects) $(DOCS) core
+
+distclean: clean
+mostlyclean: clean
+maintainer-clean: clean
+
+check: all
+	./mandos --check
+
+run-client: all
+	./mandos-client --plugin-dir=plugins.d --options-for=password-request:--keydir=keydir
+
+run-server: all
+	./mandos --debug --configdir=.

=== added file 'TODO'
--- TODO	1970-01-01 00:00:00 +0000
+++ TODO	2008-08-09 04:42:39 +0000
@@ -0,0 +1,58 @@
+-*- org -*-
+
+* README file
+
+* COPYING file
+  [[file:/usr/share/common-licenses/GPL-3][GPLv3]]
+
+* Mandos-client
+** [#A] Man page: man8/mandos-client.8mandos
+** use strsep instead of strtok?
+** use config file in addition to arguments
+** pass things in environment, like device name, etc
+
+* Password-request
+** [#A] Man page: man8/password-request.8mandos
+** [#A] check return codes of all system calls
+** [#B] header files/symbols tally
+** IPv4 support
+** use strsep instead of strtok?
+** Do not depend on GPG key rings on disk
+   This would mean creating new GPG key rings with GPGME by importing
+   the key files from scratch on every program start.
+
+* Password-prompt
+** [#A] Man page: man8/password-prompt.8mandos
+** Use getpass(3)?
+   [[info:libc:getpass][GNU LibC Manual: Reading Passwords]]
+
+* Server
+** [#A] Command man page: man8/mandos.8
+** [#A] Config file man page: man5/mandos.conf (mandos.conf)
+** [#A] Config file man page: man5/mandos-clients.conf (clients.conf)
+** [#A] write PID file
+** [#A] /etc/init.d/mandos-server
+** Better comments in config files
+** Log level
+** /etc/mandos/clients.d/*.conf
+   Watch this directory and add/remove/update clients?
+** config for TXT record
+** Run-time communication with server
+   probably using D-Bus
+** Implement --foreground
+   [[info:standards:Option%20Table][Table of Long Options]]
+** Implement --socket
+   [[info:standards:Option%20Table][Table of Long Options]]
+
+* Mandos-tools/utilities
+  All of this probably using D-Bus
+** List clients
+** Enable client
+** Disable client
+
+* Installer
+** [#A] Change initrd.img file to not be publically readable
+** [#A] Create GPG key ring files in initrd
+
+
+#+STARTUP: showall

=== removed file 'ca.pem'
--- ca.pem	2007-10-20 21:38:25 +0000
+++ ca.pem	1970-01-01 00:00:00 +0000
@@ -1,40 +0,0 @@
------BEGIN CERTIFICATE-----
-MIIHEzCCBPugAwIBAgIJAMCSQPxm3Tm7MA0GCSqGSIb3DQEBBQUAMIGMMQswCQYD
-VQQGEwJTRTELMAkGA1UECBMCQkwxEDAOBgNVBAcTB1Jvbm5lYnkxITAfBgNVBAoT
-GEludGVybmV0IFdpZGdpdHMgUHR5IEx0ZDEWMBQGA1UEAxMNQnJheGVuIHVuaXRl
-ZDEjMCEGCSqGSIb3DQEJARYUYmVsb3JuQGZ1a3QuYnNuZXQuc2UwHhcNMDcxMDE1
-MTczNDQ1WhcNMTcxMDEyMTczNDQ1WjCBjDELMAkGA1UEBhMCU0UxCzAJBgNVBAgT
-AkJMMRAwDgYDVQQHEwdSb25uZWJ5MSEwHwYDVQQKExhJbnRlcm5ldCBXaWRnaXRz
-IFB0eSBMdGQxFjAUBgNVBAMTDUJyYXhlbiB1bml0ZWQxIzAhBgkqhkiG9w0BCQEW
-FGJlbG9ybkBmdWt0LmJzbmV0LnNlMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIIC
-CgKCAgEAxrMpGUiDUI0X3XgYovgRGaKaSIoCL7RlNVRHxWLEaNIEmHuhQv40fksa
-ndO7KcbxBRTMvF/XNYyEEyks+D3jQrBmWBjsOSdZkLhP64HLnx2eYLd1HEAXIhB8
-wip180m9tfT6XeXX6cOh5AowMOGHv364xcZXJvYJgxVuMW1vlh9F79N1bnnR2rJ6
-4nmebJ91QW7ecSwA6h0fnx6GF3d5PfEC24/Fys/1NbdYiy8EYsP+lIEHQC4oD0Za
-kjclNWT3O1I8lnLdm4F1KdgDIva7aCyoAI4rLsxFxnnaqQcPo+8gGhusfnbOSymd
-m2tdz91x3Z2rSoo/cgc2TaPYKYuiJScsw1FftYGOZLiXbAedCqmoEC9XdMlZrbPn
-AWhjSvbFY8B2LCoy23OFpV2h+QrnKXOzsxyfxexwkjytIn9Msy5DlcCQz93vO/c9
-CGN/GpXJf5ebNk4oTE7pXNMM27shvubtXa6BL7A+S7yB8xmQ3NA+CABsQsKHcgVy
-2+SLD65Ms6OpwXn9BPlT1aJxbTiCXlSIQ7OIDPE1okwjxLTHc9dygoYuqjLG9Dkx
-WOqZWNxyj3kU40mX96qbOhJG5Zgq9J9fUT4ZJETO754YIFGKltlT6RSqeLrNOuLg
-9cYNGOLqoCVIyUi5HiTR7iIukNDeZy4YN6BiQH+TnYjC5+FwZbsCAwEAAaOCAXQw
-ggFwMB0GA1UdDgQWBBT4/xxxSVgFPLkDVNc8Ocdvodu3JTCBwQYDVR0jBIG5MIG2
-gBT4/xxxSVgFPLkDVNc8Ocdvodu3JaGBkqSBjzCBjDELMAkGA1UEBhMCU0UxCzAJ
-BgNVBAgTAkJMMRAwDgYDVQQHEwdSb25uZWJ5MSEwHwYDVQQKExhJbnRlcm5ldCBX
-aWRnaXRzIFB0eSBMdGQxFjAUBgNVBAMTDUJyYXhlbiB1bml0ZWQxIzAhBgkqhkiG
-9w0BCQEWFGJlbG9ybkBmdWt0LmJzbmV0LnNlggkAwJJA/GbdObswDwYDVR0TAQH/
-BAUwAwEB/zARBglghkgBhvhCAQEEBAMCAQYwCQYDVR0SBAIwADArBglghkgBhvhC
-AQ0EHhYcVGlueUNBIEdlbmVyYXRlZCBDZXJ0aWZpY2F0ZTAfBgNVHREEGDAWgRRi
-ZWxvcm5AZnVrdC5ic25ldC5zZTAOBgNVHQ8BAf8EBAMCAQYwDQYJKoZIhvcNAQEF
-BQADggIBAD/AJL0dKb4CcVEBzYsKV5JAxZQBTFYXm4TlMzbNJnl6C3FH636xSfvE
-qWWPrIplbmNn0tyXNf1dvb2Xy4QVFTo5lnp6PKwhqpV5JK7+4/LiDZj5R4VNl36B
-RUleXSSGV/+PJ1hRC9J/s+oziK1025hPpDB+n13sFrBv9DyqP/3GDSVoi13Mldf1
-vTWRnvugi7XlMhFdemB1Vz0eyIde0FtFpPrd3HeFrXQX8617XfcEYJmCU2aDT1fy
-77RHFG+3dL286UcK2D85bqNCntP9W3zP6rwerkO1pGmT2cBn9AO6Qt2MI7klTJmw
-SRqbeBcF+gaBRGY8iZHcqow7vqTvo65p/ZH9AUqMQh41W40FbepUA3354xGmwAlB
-LBXRuVjaDvKyargh936dOyzIDr+fZtr6BEthmt6BghgOZlezAV/P60P56DlFvpK8
-4wHyT5KI9LOqbg+W2ToB/GdRZz4p7K90JljrgVlnQhYIQ1YT/5OzPu6Uq5+4Q4hc
-ORb5hRCWHwre95kgEzUPqrteqV/g99XaOgqpK6KhY/dpmb7Z5I9f6d61XYYFfxYm
-iuNJiTLCQlLyfXFSRlJubWD8hNHo2HNRYdiWF3cttuGL9aC0uSIk4xq/Oi/guqIn
-sZlrO3I4MaiUnUXntMLU4zh0o/RPqhoT3lY1VubwgmpkOw8p7ggE
------END CERTIFICATE-----

=== removed file 'cert.pem'
--- cert.pem	2007-10-20 21:38:25 +0000
+++ cert.pem	1970-01-01 00:00:00 +0000
@@ -1,40 +0,0 @@
------BEGIN CERTIFICATE-----
-MIIG8zCCBNugAwIBAgIBATANBgkqhkiG9w0BAQUFADCBjDELMAkGA1UEBhMCU0Ux
-CzAJBgNVBAgTAkJMMRAwDgYDVQQHEwdSb25uZWJ5MSEwHwYDVQQKExhJbnRlcm5l
-dCBXaWRnaXRzIFB0eSBMdGQxFjAUBgNVBAMTDUJyYXhlbiB1bml0ZWQxIzAhBgkq
-hkiG9w0BCQEWFGJlbG9ybkBmdWt0LmJzbmV0LnNlMB4XDTA3MTAxNTE3MzczMVoX
-DTA4MTAxNDE3MzczMVowdTELMAkGA1UEBhMCU0UxCzAJBgNVBAgTAkJMMRAwDgYD
-VQQHEwdSb25uZWJ5MREwDwYDVQQKEwhHbnVzdHVmZjEPMA0GA1UEAxMGQnJheGVu
-MSMwIQYJKoZIhvcNAQkBFhRiZWxvcm5AZnVrdC5ic25ldC5zZTCCAiIwDQYJKoZI
-hvcNAQEBBQADggIPADCCAgoCggIBAK4j59iJGfir5jxl4gCeLyhXED4YGd42o/ll
-XdWtYO9IHtrCZknnRIjjviQ0PJk6JQpIGkNrdW5uRSQcTKshcEOL8cprUHe+Mpi8
-34i7bOaPlDdJYoMatkHEvq92weXFfKa8yG5UtmAbiuo388JbnVgU7HAqq1ipi+dS
-MoJU0FR3qxKC3eioQQdq+QB257BeZNDq3SrWTqZXw7pu4DmiOZJhpMAV5CmjQKt4
-ZC0EdByGct0tp3X3swpBY+0a+wlFMKVe1WGy9dfhkNRVhUCXNKOCuaaIMsn8iuCc
-NlkIw4GVhvJj9ALfRZS5VzRaKIhOnFub4+PxYClt1ghfTBhk1aQjiSl1JT+qtgpv
-Sqd34OnYtLRp+L5EcLQSPoykG2TjsUxnYXtBiPoZuIk7v5alQlbT20DmuWhpCMm4
-fw1W0Q6eXL2APtrjAt7HS79qDopySL4st8nTkY3fzmjW8Vpg1Y4HyN6aY8tR9OIj
-SwZhCv+NoixbZToUnLXmoRtQnO6rSp7qLuJCPC1gbDc1TR4mclr5Vumb8ag1iQTi
-i0SvhqY8u1sDMGlL9aZW0WaykfoC64cUdBKm9R6dt08Y0xAevSaxdcsmK9kCfCJO
-o6nJ2pQ9+WdSjYEhq5DLBGhpfhzs+U1ZL0HYmTfyVjmbWuDZaQ1Yht4vHf1TYwAp
-FjGJGt/7AgMBAAGjggF0MIIBcDAJBgNVHRMEAjAAMBEGCWCGSAGG+EIBAQQEAwIG
-QDArBglghkgBhvhCAQ0EHhYcVGlueUNBIEdlbmVyYXRlZCBDZXJ0aWZpY2F0ZTAd
-BgNVHQ4EFgQUoq4JMCRt6ToFRTrLNUFd4xQ0hGUwgcEGA1UdIwSBuTCBtoAU+P8c
-cUlYBTy5A1TXPDnHb6HbtyWhgZKkgY8wgYwxCzAJBgNVBAYTAlNFMQswCQYDVQQI
-EwJCTDEQMA4GA1UEBxMHUm9ubmVieTEhMB8GA1UEChMYSW50ZXJuZXQgV2lkZ2l0
-cyBQdHkgTHRkMRYwFAYDVQQDEw1CcmF4ZW4gdW5pdGVkMSMwIQYJKoZIhvcNAQkB
-FhRiZWxvcm5AZnVrdC5ic25ldC5zZYIJAMCSQPxm3Tm7MB8GA1UdEgQYMBaBFGJl
-bG9ybkBmdWt0LmJzbmV0LnNlMB8GA1UdEQQYMBaBFGJlbG9ybkBmdWt0LmJzbmV0
-LnNlMA0GCSqGSIb3DQEBBQUAA4ICAQBC5Hz334l9o0RnjjjzZZyfYpEuAK7MJr6g
-r7hPqZzAAXL7i4w5AEutthIxo+JGGL0P4xv2Swc6uwodU0GE4/6DUEBQrjaDhr/F
-Uw3GaXYVopdDa/kYWAXF6lP+hkNhb1hI5onLMtRpoICLpfePNALZn3lMaBPI2efC
-EvViu3dpjJKKNm/HlXOl/wgrrcLwHKlbPSozaiXe2qxik4fr15cXtm4/nXetZq1g
-8NAOcZpdurZNEieMhtFSvLbQ3X/yNYKLgukz/zVCzL0IXLQXA4B2URBbzQJtrjOD
-Agq5mAXhrlGuEEvaMJeQP+VrfsIZU2fFKvr8LVQO37GprQizPFBC8FDkrE4XrpET
-A6ztDijyWRxT5x+4MbZrfL3GE9ZO57HZaAdXil1cjxXMlFqtJ6yoLNil3v+mgp9G
-yJeEz6L3jYRxVZqXYy7VMh+rJ7Mdr31g9pT8TT0/LGoCBorCTcUYIBP3BE3F85Nk
-JCPR5GG7Mfk3s8VTxTFAMZGKgst+i+YiMQSLkXIHNph5Fi18qIacpD9kkCXVj+JU
-UW/fYcTd4vQdR2HhyJdPddtHkprMN3ZlICEug3junMOJb0PWByqfuaihkPYIZ2xw
-puEkrP8cMdTXiUXep2RHTwByqy3/EQMC3ZWRiMIHZRIkl+o5bklOHwadUmzrJ1vL
-QjmtPc7y1g==
------END CERTIFICATE-----

=== removed file 'client-cert.pem'
--- client-cert.pem	2007-10-20 21:38:25 +0000
+++ client-cert.pem	1970-01-01 00:00:00 +0000
@@ -1,40 +0,0 @@
------BEGIN CERTIFICATE-----
-MIIHBzCCBO+gAwIBAgIBAjANBgkqhkiG9w0BAQUFADCBjDELMAkGA1UEBhMCU0Ux
-CzAJBgNVBAgTAkJMMRAwDgYDVQQHEwdSb25uZWJ5MSEwHwYDVQQKExhJbnRlcm5l
-dCBXaWRnaXRzIFB0eSBMdGQxFjAUBgNVBAMTDUJyYXhlbiB1bml0ZWQxIzAhBgkq
-hkiG9w0BCQEWFGJlbG9ybkBmdWt0LmJzbmV0LnNlMB4XDTA3MTAxNTIwNDkwNloX
-DTA4MTAxNDIwNDkwNlowfDELMAkGA1UEBhMCU0UxCzAJBgNVBAgTAkJMMRAwDgYD
-VQQHEwdSb25uZWJ5MREwDwYDVQQKEwhnbnVzdHVmZjEWMBQGA1UEAxQNYnJheGVu
-X2NsaWVudDEjMCEGCSqGSIb3DQEJARYUYmVsb3JuQGZ1a3QuYnNuZXQuc2UwggIi
-MA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQDWAwsXT4jeKCKg7/4g8KTmwEZ6
-88zfaJeqpzpkpc25J47Qmr5Mf6NiKUIAqjD8KycApG1cwVoLEvxvQOip2lSzL4ex
-ipTpZO45O6N5blKG4XKqJRE2VE5N1j2Z+jJmfFEaHqMDYb9+dlnSZ2c0c1d2qcgC
-HgOR5iZHWQCGtEK3y9HfWsFpULuzfYMIpYZjPic+kAuisT5cl/wIDVI/MbpB3/vk
-9vMpRCFRjvbOcN7rEW4UihuVSFmzFNk99QR8e3Li1HtqbXmmtJps9u16ugGQO4ED
-4gk65i5TU15sZJgFLXBuFuZFYH8WF2CIu6o4oylCLVjFRZHl8jKwZdPtNUZ01cJk
-oj+M9Yc9m6F1dLafLThhvSLMuRfUvHY9bFAkF6QkkgIjY1mN1VTm6XxLN991DJd7
-67IfpFrBaKi47C4abBc4lZYfBlmXdkpOt3UcYxh3YT+KTUK/7S/arj2iEypTic4x
-PVDsA4DbnHJ1+6ZzyMAM7ACmZnWhq7gXhjT8z3eEyaqaZVOracG1DFln781Mm6b9
-Eb3qJfczKR/QXiMsF4gwwH+Dclkd/THjjv/SPQYCiWm2Fo8IXG/hdZzBx4dWoSMZ
-kgQM5GDLQU80/jAvDni6n97I48fcxjL6YnEgQeoasKoayGaOmnhEs/eytpcGocNi
-3pJUpD0coYyAemeL5wIDAQABo4IBgTCCAX0wCQYDVR0TBAIwADARBglghkgBhvhC
-AQEEBAMCBLAwKwYJYIZIAYb4QgENBB4WHFRpbnlDQSBHZW5lcmF0ZWQgQ2VydGlm
-aWNhdGUwHQYDVR0OBBYEFK9FlbSWMi9o063eYNprvQ4Msg/FMIHBBgNVHSMEgbkw
-gbaAFPj/HHFJWAU8uQNU1zw5x2+h27cloYGSpIGPMIGMMQswCQYDVQQGEwJTRTEL
-MAkGA1UECBMCQkwxEDAOBgNVBAcTB1Jvbm5lYnkxITAfBgNVBAoTGEludGVybmV0
-IFdpZGdpdHMgUHR5IEx0ZDEWMBQGA1UEAxMNQnJheGVuIHVuaXRlZDEjMCEGCSqG
-SIb3DQEJARYUYmVsb3JuQGZ1a3QuYnNuZXQuc2WCCQDAkkD8Zt05uzAfBgNVHRIE
-GDAWgRRiZWxvcm5AZnVrdC5ic25ldC5zZTAfBgNVHREEGDAWgRRiZWxvcm5AZnVr
-dC5ic25ldC5zZTALBgNVHQ8EBAMCBaAwDQYJKoZIhvcNAQEFBQADggIBAINB3d3z
-FDRV5/+vov3KhyKhhDYdhO2IqkSXARRQffrd94/F/8YcrOgJJrvnz6THbQIzuSM7
-sSE2Y2JqyWfaA0J8CQm9JrOgVTE9I74/IZONC3jSBrt/Q9dG74otaeaHNOTaH4lU
-nAH5Cv6CgMFi58n+yQLxrYBAimWx8x1u40OxUXM+AbpAgDo3uXNt62SdrWAldwh3
-9Eqog3qUcN7R1VGx535VXJGg3e8yZFRPnFLiYuO1afjO/tlOyYBG0wvCfmNMSpcx
-+/ycIXzTH5kvIcPoa8oCaAuCMgtsMZ2SoBJ37xgwb1qKjfVAOfju1qVYgI5jqQLp
-sNsS4FqyS7jpKoUxbcShvOui5GUunXgxZ/VMd7rqSqbC8ozKXvOEYaJWf9lY7DCR
-RdFR03CmWZ8EDO2JM1sKRni9gVz7wflSPjI8vDw9EbQzrW0Du64i4aBC2/kuJq9g
-7rEQrjMRh8/kQigCP1BUk1l+r/gPO8UeIBc7tTls0FsPIkKQq7cPNQgNiNIIV8v2
-iLW1bZmT2vswpiyqbz7LtGzYj+x1Ce+7U9iXgY08QgbiqiXl/eNw/xNVkFaMMDc2
-4+APQMF790OaZvWtme9XzePsy16wLgF4G/Xh1M8RfG3z7vXXm4Vl38iErZkXTmMl
-63Z87xA2LHvcoMEpP/nJHnvts3X2o3u7FFG9
------END CERTIFICATE-----

=== removed file 'client-key.pem'
--- client-key.pem	2007-10-20 21:38:25 +0000
+++ client-key.pem	1970-01-01 00:00:00 +0000
@@ -1,51 +0,0 @@
------BEGIN RSA PRIVATE KEY-----
-MIIJKgIBAAKCAgEA1gMLF0+I3igioO/+IPCk5sBGevPM32iXqqc6ZKXNuSeO0Jq+
-TH+jYilCAKow/CsnAKRtXMFaCxL8b0DoqdpUsy+HsYqU6WTuOTujeW5ShuFyqiUR
-NlROTdY9mfoyZnxRGh6jA2G/fnZZ0mdnNHNXdqnIAh4DkeYmR1kAhrRCt8vR31rB
-aVC7s32DCKWGYz4nPpALorE+XJf8CA1SPzG6Qd/75PbzKUQhUY72znDe6xFuFIob
-lUhZsxTZPfUEfHty4tR7am15prSabPbteroBkDuBA+IJOuYuU1NebGSYBS1wbhbm
-RWB/FhdgiLuqOKMpQi1YxUWR5fIysGXT7TVGdNXCZKI/jPWHPZuhdXS2ny04Yb0i
-zLkX1Lx2PWxQJBekJJICI2NZjdVU5ul8SzffdQyXe+uyH6RawWiouOwuGmwXOJWW
-HwZZl3ZKTrd1HGMYd2E/ik1Cv+0v2q49ohMqU4nOMT1Q7AOA25xydfumc8jADOwA
-pmZ1oau4F4Y0/M93hMmqmmVTq2nBtQxZZ+/NTJum/RG96iX3Mykf0F4jLBeIMMB/
-g3JZHf0x447/0j0GAolpthaPCFxv4XWcwceHVqEjGZIEDORgy0FPNP4wLw54up/e
-yOPH3MYy+mJxIEHqGrCqGshmjpp4RLP3sraXBqHDYt6SVKQ9HKGMgHpni+cCAwEA
-AQKCAgBha4c7+EecoXaJ/lWXlxPpurMauyqStGD+HRvWvycz1s8LJLXlyuCMCa3y
-8YZU9CvP/gmOhLHBgsYIuupuj2WpH8TMTAJXcEuFICHdYBwPLEdvLmp0adIvWow2
-MI+K2aJtmm6oVnG+Vo+y2MFBPhQdf1H9rL4BR1w7dEdqClqoog6Kdxy+HTMklMj2
-Qas4OA3TS+0QBVEXA1SGMdIz1CYuYJCg/M1aBpqILuUounavWQLcNLYzsXirrZzq
-uENvix6UJRd9LhKHkYUOfyVBjbSyfHPRWa7L8gY6hiPggbY1/SZF5wSxpiiT3NZj
-x9HH8HYSmuPjATVWEHeElwXu4CaOqax6OCMUwrIHWXUq/ydWjDXk/vK6PU1d2Fvh
-Y274QkBHwrDRZ7HiDLZDO6VVJx9O0WKajkLqidVMFzSZem+rxlIW8KwJUto8ObEi
-DK6VfkD0f1VfhWbVyb4dzgmloseHc9S/31ZBY4MfESpgVOh3rg12eXl4BVTRzDlf
-Qq0Z9hKuBaURt/CNv+ALpAp+u6mvzmNuSnjuXOkD3bmYwPZljykf1A3Cr8eyPK1E
-4PbLZN6Te7pbbjDvj+k23dzk1HUZ+I+5ykJi1v7tsQIiUOxFVJRMEmsfG+rMmKp6
-+9fsgcqK0wpnZUJ325ghjaaGR4i4AUMYVHn+mTmbhCDis97Q2QKCAQEA71FMbFGt
-pBzYOCn9hMrBDIAU2tQNkmukeg7HremIyh5zm3Lt9YdSNGM/D1WqN3XdsdVczLoy
-l89jUTyoqoFQkackTkPxftUd6655CuOqSGFhr8xVk7SSvC9ZbwV69WgVJkYfrYv5
-3gXPAmBgc3S8S400ik5AtKh3XIQMsWwKPdpHKQZvbpyw851IoRQEzUhiT/Bi3Uug
-PzZHyFNmJMrPqSaot73zwkmkPK3KOCgAxStB/Ab2JNx8yjfL2UxIqunU5TaO65Wp
-2Be2mSPwYe0FZ8oE4HPKmOWJC7jHgT88OzdaeT9jBTOecGbvm/Wz2Rr83wUkMZlf
-+/sF2XOrIsCSjQKCAQEA5O4oDf+t0vvEonTC9YE5bQ/rkSz4SKYI23nnI+u19C4E
-ntsHzrU9VMrnZV/Zc+j+z5hObgN179ePTvHLSjhvfo1zONGznl05YZRUE0dLf9bw
-ns3tNjxYc86BtYGGJfX8WqgsTLCTotrRY8hk2D2NupzB21WfaalWi2pZEsEq5vk4
-gqh7IwraS7x56GQTQIPfdQmv9XNfDh7VO4bcaqAI1n+KIpJTFVcEHnu6RpoQyE12
-F2oEQOo/S4nY2N+MZioavO46o9r4p/3V8CyuXQghkal2GHZeCkTpMto1BayHYOKc
-ZwxxLV/F/Nhr2BqJwiUojvbDcjcDeZVysjHQfNI1QwKCAQEAwwbd4OgvOa7IBf0y
-PSV+bVFzrWFiLhDK2S1yTKgkcZKfY+8lPRIqS8cVfMmzDb9gC6x1E+IpmM3JgkqM
-qWb44bn0PFPiWhoTaB2nRtiBzLqPgVRj6Rse+X0cxP0SVyubELXU2vlXhzf0m1sv
-PufDC07nok5jLNadbyetsGj7b6ySkTxNUzcefWmP5rUJtMFoXPzplK4syVbS6M4O
-T613zcFTfWmvXIXm8gwu39S5y/SUsW566U9F4wXVeiBQl/g3JxRvJE2zPAcXJ3XC
-UAt4fDyF/ORgFnn4VTUgYJPH4foaIPUnHPYUCEXavp5dEnCL4rOt6z2ymwbWnX9F
-1+xXgQKCAQEAr1XAE8ipRxhRJ+OplgKdCuzQjOYWWv3fdslMwHQ9bYD5RPmYZzbk
-fFbTFw9sKpxe3HxYRWYdI87DEcGa44OJ0TFg+DmUCkx4MEY8hm9qYcUrkVVCwvFB
-BaE6Mtu69MQLvRtkom/zAx34lSXcJsouDKkWyHgxmel6QVj4U6bixvhF0bxcYyBi
-xPLbo6NSI21c7fS3cZQlT1lKE0dc9cUQP8h68bOEMqnsm7RDnONOyzqYoaIvE9DV
-HXO8Q44jp/PDesQy4WBKZc/B1StDeDlMDJXvvxiZOeBBgxMg3PGg1hF5nGspG4lo
-yBixsFfS/oEbKTPRyV5dKPfPWq9QcOlGpQKCAQEAwe2qtx7tzI7cdx1JLSHNZyPp
-h9/WTjYKVg3Sl6dXfDsNTqIUkRrwpB1tEZXs1PIq5641ev4LmRxGDPLJWIwNZ1LN
-slR12towc6LTY/05OIBHrTIAW8fGs2E4YcqYcLWsWuzU9BsJLQeBdUth9zF6qZoK
-USN6a3326dx8lhUcQ4zJ9c/DtK+99WAXtt0Cki7aICvLbcsmRbKPHpJyzRpDI7uH
-Ex/UgjSR1w5L2Seou3e6ljxYGAecqa86HIkHVleDIrIYl6GlKDBKSVzfmLh/97mi
-o3oTNqRal934xYS6ZTId/ZkRL5c78U/NJlK1H5gYzfL65mJ1+c2N3cR3sy4jtQ==
------END RSA PRIVATE KEY-----

=== removed file 'client.cpp'
--- client.cpp	2008-01-18 21:18:26 +0000
+++ client.cpp	1970-01-01 00:00:00 +0000
@@ -1,209 +0,0 @@
-extern "C" {
-#include <sys/types.h>		// getaddrinfo, gai_strerror, socket, inet_pton
-				// connect
-#include <sys/socket.h>		// getaddrinfo, gai_strerror, socket, inet_pton
-				// connect
-#include <unistd.h>		// close, STDIN_FILENO, STDOUT_FILENO
-#include <netdb.h>		// getaddrinfo, gai_strerror
-#include <arpa/inet.h>		// inet_pton
-#include <sys/select.h> 	// select
-#include <gnutls/gnutls.h>
-#include <sys/ioctl.h> 		// ioctl, ifreq, SIOCGIFFLAGS, IFF_UP, SIOCSIFFLAGS
-#include <net/if.h> 		// ioctl, ifreq, SIOCGIFFLAGS, IFF_UP, SIOCSIFFLAGS
-#include <termios.h> 		// struct termios, tcsetattr, tcgetattr, TCSAFLUSH, ECHO
-}
-
-#include <cerrno>		// perror
-#include <cstring> 		// memset
-#include <string> 		// std::string, std::getline
-#include <iostream> 		// cin, cout, cerr
-#include <ostream> 		// <<
-
-#define SOCKET_ERR(err,s) if(err<0) {perror(s); status = 1; goto quit;}
-#define PORT 49001
-#define CERTFILE "/conf/conf.d/cryptkeyreq/client-cert.pem"
-#define KEYFILE "/conf/conf.d/cryptkeyreq/client-key.pem"
-#define CAFILE "/conf/conf.d/cryptkeyreq/ca.pem"
-
-gnutls_certificate_credentials_t x509_cred;
-
-gnutls_session_t
-initgnutls(){
-  gnutls_session_t session;
-
-#ifdef DEBUG
-  std::cerr << "Initiate certificates\n";
-#endif
-
-  gnutls_global_init ();
-
-  /* X509 stuff */
-  gnutls_certificate_allocate_credentials (&x509_cred);
-  gnutls_certificate_set_x509_trust_file (x509_cred, CAFILE, GNUTLS_X509_FMT_PEM);
-  gnutls_certificate_set_x509_key_file (x509_cred, CERTFILE, KEYFILE,
-					GNUTLS_X509_FMT_PEM);
-
-  //Gnutls stuff
-  gnutls_init (&session, GNUTLS_CLIENT);
-  gnutls_set_default_priority (session);
-  gnutls_credentials_set (session, GNUTLS_CRD_CERTIFICATE, x509_cred);
-  return session;
-}
-
-
-int main (){
-  int udp_sd, tcp_sd, ret;
-  char buffer[4096];
-  struct sockaddr_in6 to;
-  struct sockaddr_in6 from;
-  gnutls_session_t session;
-  fd_set rfds_orig;
-  struct timeval timeout;
-
-  struct termios t_old, t_new;
-  int status = 0;
-
-  session = initgnutls ();
-
-#ifdef DEBUG
-  std::cerr << "Open ipv6 UDP\n";
-#endif
-
-  udp_sd = socket(PF_INET6, SOCK_DGRAM, 0);
-  SOCKET_ERR(udp_sd,"socket");
-  
-#ifdef DEBUG
-  std::cerr << "Open socket with socket nr: " << udp_sd << '\n';
-#endif
-  
-  {
-    int flag = 1;
-    ret = setsockopt(udp_sd, SOL_SOCKET, SO_BROADCAST, & flag, sizeof(flag));
-    SOCKET_ERR(ret,"setsockopt broadcast");
-  }
-
-  ret = setsockopt(udp_sd, SOL_SOCKET, SO_BINDTODEVICE, "eth0", 5);
-  SOCKET_ERR(ret,"setsockopt bindtodevice");
-
-  memset (&to, '\0', sizeof (to));
-  to.sin6_family = AF_INET6;
-  ret = inet_pton(AF_INET6, "ff02::1" , &to.sin6_addr);
-  SOCKET_ERR(ret,"inet_pton");
-  to.sin6_port = htons (PORT);	// Server Port number
-
-  struct ifreq network;
-
-  strcpy(network.ifr_name, "eth0");
-
-  ret = ioctl(udp_sd, SIOCGIFFLAGS, &network);
-  SOCKET_ERR(ret,"ioctl SIOCGIFFLAGS");
-
-  network.ifr_flags |= IFF_UP;
-
-  ret = ioctl(udp_sd, SIOCSIFFLAGS, &network);
-  SOCKET_ERR(ret,"ioctl SIOCSIFFLAGS");
-    
-  FD_ZERO(&rfds_orig);
-  FD_SET(udp_sd, &rfds_orig);
-  FD_SET(STDIN_FILENO, &rfds_orig);
-
-
-  if (tcgetattr (STDIN_FILENO, &t_old) != 0){
-    return 1;
-  }
-  t_new = t_old;
-  t_new.c_lflag &= ~ECHO;
-  if (tcsetattr (STDIN_FILENO, TCSAFLUSH, &t_new) != 0){
-    return 1;
-  }
-  
-  for(;;){
-    for(;;){
-
-#ifdef DEBUG
-      std::cerr << "Sending Marco on UDP\n";
-#endif
-      ret = sendto(udp_sd, "Marco", 5, 0, reinterpret_cast<const sockaddr*>(&to), sizeof(to));
-      if (ret < 0){
-	perror("sendto");
-      }
-      
-      fd_set rfds = rfds_orig;
-      timeout.tv_sec = 10;
-      timeout.tv_usec = 0;
-      
-      std::cerr << "Password: ";
-      
-      ret = select(udp_sd+1, &rfds, 0, 0, & timeout);
-      SOCKET_ERR(udp_sd,"select");
-      
-      if (ret){
-	if (FD_ISSET(STDIN_FILENO, &rfds)){
-	  std::string buffer;
-	  std::getline(std::cin, buffer);
-	  std::cerr << '\n';
-	  std::cout << buffer;
-	  goto quit;
-	}
-	
-	socklen_t from_len = sizeof(from);
-	ret = recvfrom(udp_sd,buffer,512,0, reinterpret_cast<sockaddr *>(& from),
-		       & from_len);
-	SOCKET_ERR(ret,"recv");
-	
-	if (strncmp(buffer,"Polo", 4) == 0){
-	  break;
-	}
-      }
-      std::cerr << '\r';
-    }
-    
-    
-    tcp_sd = socket(PF_INET6, SOCK_STREAM, 0);
-    SOCKET_ERR(tcp_sd,"socket");
-    
-    setsockopt(tcp_sd, SOL_SOCKET, SO_BINDTODEVICE, "eth0", 5);
-    SOCKET_ERR(ret,"setsockopt bindtodevice");
-    
-    memset(&to,0,sizeof(to));
-    to.sin6_family = from.sin6_family;
-    to.sin6_port   = from.sin6_port;
-    to.sin6_addr   = from.sin6_addr;
-    to.sin6_scope_id   = from.sin6_scope_id;
-    
-    ret = connect(tcp_sd,reinterpret_cast<struct sockaddr *>(&to),sizeof(to));
-    if (ret < 0){
-      perror("connect");
-      continue;
-    }
-    
-    gnutls_transport_set_ptr (session, reinterpret_cast<gnutls_transport_ptr_t> (tcp_sd));
-    
-    ret = gnutls_handshake (session);
-    
-    if (ret < 0)
-      {
-	std::cerr << "\n*** Handshake failed ***\n";
-	gnutls_perror (ret);
-	continue;
-      }
-    
-    //retrive password
-    ret = gnutls_record_recv (session, buffer, sizeof(buffer));
-    
-    write(STDOUT_FILENO,buffer,ret);
-    
-    //shutdown procedure
-    gnutls_bye (session, GNUTLS_SHUT_RDWR);
-    close(tcp_sd);
-    gnutls_deinit (session);
-    gnutls_certificate_free_credentials (x509_cred);
-    gnutls_global_deinit ();
-    break;
-  }
-  close(udp_sd);
-
- quit:
-  tcsetattr (STDIN_FILENO, TCSAFLUSH, &t_old);
-  return status;
-}

=== added file 'clients.conf'
--- clients.conf	1970-01-01 00:00:00 +0000
+++ clients.conf	2008-08-08 01:51:58 +0000
@@ -0,0 +1,35 @@
+[DEFAULT]
+timeout = 1h
+interval = 5m
+checker = fping -q -- %%(fqdn)s
+
+# Example
+[foo]
+fingerprint = 3e393aeaefb84c7e89e2f547b3a107558fca3a27
+secret = Base+64+encoded+OpenPGP+encrypted+data/=
+# secfile = /etc/mandos/foo-secret.txt.asc
+fqdn = foo.example.org
+checker = fping -q -- %%(fqdn)s
+timeout = 10m
+
+[braxen_client]
+fingerprint =  7788 2722 5BA7 DE53 9C5A  7CFA 59CF F7CD BD9A 5920
+secret =
+        hQIOA6QdEjBs2L/HEAf/TCyrDe5Xnm9esa+Pb/vWF9CUqfn4srzVgSu234
+        REJMVv7lBSrPE2132Lmd2gqF1HeLKDJRSVxJpt6xoWOChGHg+TMyXDxK+N
+        Xl89vGvdU1XfhKkVm9MDLOgT5ECDPysDGHFPDhqHOSu3Kaw2DWMV/iH9vz
+        3Z20erVNbdcvyBnuojcoWO/6yfB5EQO0BXp7kcyy00USA3CjD5FGZdoQGI
+        Tb8A/ar0tVA5crSQmaSotm6KmNLhrFnZ5BxX+TiE+eTUTqSloWRY6VAvqW
+        QHC7OASxK5E6RXPBuFH5IohUA2Qbk5AHt99pYvsIPX88j2rWauOokoiKZo
+        t/9leJ8VxO5l3wf/U64IH8bkPIoWmWZfd/nqh4uwGNbCgKMyT+AnvH7kMJ
+        3i7DivfWl2mKLV0PyPHUNva0VQxX6yYjcOhj1R6fCr/at8/NSLe2OhLchz
+        dC+Ls9h+kvJXgF8Sisv+Wk/1RadPLFmraRlqvJwt6Ww21LpiXqXHV2mIgq
+        WnR98YgSvUi3TJHrUQiNc9YyBzuRo0AjgG2C9qiE3FM+Y28+iQ/sR3+bFs
+        zYuZKVTObqiIslwXu7imO0cvvFRgJF/6u3HNFQ4LUTGhiM3FQmC6NNlF3/
+        vJM2hwRDMcJqDd54Twx90Wh+tYz0z7QMsK4ANXWHHWHR0JchnLWmenzbtW
+        5MHdW9AYsNJZAQSOpirE4Xi31CSlWAi9KV+cUCmWF5zOFy1x23P6PjdaRm
+        4T2zw4dxS5NswXWU0sVEXxjs6PYxuIiCTL7vdpx8QjBkrPWDrAbcMyBr2O
+        QlnHIvPzEArRQLo=
+        =iHhv
+fqdn = localhost
+interval = 5m

=== removed file 'crl.pem'
--- crl.pem	2007-10-20 21:38:25 +0000
+++ crl.pem	1970-01-01 00:00:00 +0000
@@ -1,18 +0,0 @@
------BEGIN X509 CRL-----
-MIIC0zCBvDANBgkqhkiG9w0BAQUFADCBjDELMAkGA1UEBhMCU0UxCzAJBgNVBAgT
-AkJMMRAwDgYDVQQHEwdSb25uZWJ5MSEwHwYDVQQKExhJbnRlcm5ldCBXaWRnaXRz
-IFB0eSBMdGQxFjAUBgNVBAMTDUJyYXhlbiB1bml0ZWQxIzAhBgkqhkiG9w0BCQEW
-FGJlbG9ybkBmdWt0LmJzbmV0LnNlFw0wNzEwMTUxNzQ2MTlaFw0wNzExMTQxNzQ2
-MTlaMA0GCSqGSIb3DQEBBQUAA4ICAQCYTCUYRfx8+lTNXsVMhLT/890agPGj7BQw
-NhwHTZuEudPTxBtOLPf0za4z7eGTD9ggu7SayQWEOV4bfjv1yiOLzf6vEHdzv1Ee
-mlLhYgDMIhACrQmfKAmjoabsaK+VccBJW/R1oNW5Z9sWjFP91+3T7lfp8pPvWAlf
-+9mJaaysd1yguY0OITAIWEL2lLlGtd85RYLvJe2nWZ6GrH5mIEYA7IQrnPgcU3ij
-eAEn88I7EofUHfn1TMpMDJgMKm/edvEerLKb62AblcGLfo4gOBQWcvCAWLPzqxhE
-wKag1xL8ucG6250yfkYBf3KEMLZAU8py5MwaIqMVOQzz3gsQ7dE87xR5GndLvPvr
-149RbKDSZDdPDOEmCqlmb/Sxppm7jsNwqAphrZlNsBgLTrxih7Ex1cFph4jA5AZ5
-Hgqpftb94CdauOPz/AVu5aeXIwG0dpxhN0dtemwhHIxslFwDtuFWwcmP3upDZUOM
-Q4ZZBp2A1lhDW86w3law8E2TCuDFmwNtqp5zOMtOwAJF2RK2DquaaxKQG0JRhdTi
-f3mzjTTRPXomBgf7U1W25RFMO91uslCijAr/ELAa9SkFYWArnbfGabDsoy3OGL8D
-fjuaz2eIdwvASyPwUkGlfeFBRDNIRCZ3szsvtThMFWUxKvKTRBTJf4AFLwx3XWOS
-R6IxSk3t0A==
------END X509 CRL-----

=== removed file 'key.pem'
--- key.pem	2007-10-20 21:38:25 +0000
+++ key.pem	1970-01-01 00:00:00 +0000
@@ -1,51 +0,0 @@
------BEGIN RSA PRIVATE KEY-----
-MIIJKAIBAAKCAgEAriPn2IkZ+KvmPGXiAJ4vKFcQPhgZ3jaj+WVd1a1g70ge2sJm
-SedEiOO+JDQ8mTolCkgaQ2t1bm5FJBxMqyFwQ4vxymtQd74ymLzfiLts5o+UN0li
-gxq2QcS+r3bB5cV8przIblS2YBuK6jfzwludWBTscCqrWKmL51IyglTQVHerEoLd
-6KhBB2r5AHbnsF5k0OrdKtZOplfDum7gOaI5kmGkwBXkKaNAq3hkLQR0HIZy3S2n
-dfezCkFj7Rr7CUUwpV7VYbL11+GQ1FWFQJc0o4K5pogyyfyK4Jw2WQjDgZWG8mP0
-At9FlLlXNFooiE6cW5vj4/FgKW3WCF9MGGTVpCOJKXUlP6q2Cm9Kp3fg6di0tGn4
-vkRwtBI+jKQbZOOxTGdhe0GI+hm4iTu/lqVCVtPbQOa5aGkIybh/DVbRDp5cvYA+
-2uMC3sdLv2oOinJIviy3ydORjd/OaNbxWmDVjgfI3ppjy1H04iNLBmEK/42iLFtl
-OhScteahG1Cc7qtKnuou4kI8LWBsNzVNHiZyWvlW6ZvxqDWJBOKLRK+Gpjy7WwMw
-aUv1plbRZrKR+gLrhxR0Eqb1Hp23TxjTEB69JrF1yyYr2QJ8Ik6jqcnalD35Z1KN
-gSGrkMsEaGl+HOz5TVkvQdiZN/JWOZta4NlpDViG3i8d/VNjACkWMYka3/sCAwEA
-AQKCAgEApV0fWvbGnOfQGOa++Ms+CNa0a+LDHctRZxElTDX6aP9ZnW8hZ2igIkXy
-V7rrGK6oYd0aY4910koQijv9ajy0uM/56biCj1MkBPrGYrdosIEDxISBcfI5xLaq
-RUFG24Tv2/5FbtAu55EAF27OoXASOISWCeXbFLTcT+w0XqNfufZxk4CGbdro9bxV
-fGVtmoPoxKNjJryfr0KEcVO8xb4RYborkuS25/tI/Au1RTKHeFcMWJB0B4gSktiJ
-pa0LWkBD86Xch4xD/J8MwzX69d0gdW08ErIfWMPitWxiB6ZU0YdGwIK+QsP29UYT
-U3mSJ+5OeIfOnrSmFnFg4E5umnOWk1fCkX9FpFwjFy2H9FrF/BHyZrIbv8IeKgzw
-mySRXFl2KZUk8r8YyCTm4dtf2ZCU5KcUo9TOae1VzywjXFyV+AyOtFYcJ+FjHQag
-CjuT4NzmFqlwa4JWXsAtnom7eR+f4jpn3nPW2CYCE/tYxtF9fpGXdXCNaKo8fbsA
-FzOkvoPSAeOjDjcKba6Z/mMBb8or1ide06NEu77e8de8rWCEfdGbMGDmdLQaYI6u
-/WIY7ALRppqT9WKnAjKGgYkxFeZGGF8D4ARaLJYYJ71DGAKeSkK6RqHqhmWDcSI8
-cLh5WHqzCflQhxjVf5rCF/ASwTnbw0dsClqF35gw+TKAlXc/uhECggEBAOMVwVIR
-ZkCvcfH35RGxqMZ/Ca8Viqap5zLY/gL0YTM/HYhRPE8iUutCWGN1SBDQm2yKuk84
-H1Jjj7zTOxIVW0yov1m36KCsF31s8ks1b3uo+yEOyLhGpJPUq8mYAVbl/13Tcwp3
-0zs7oflZxK65vICUreJM9P/BqXJ/Edfddj88uIqjW98XGgTCgCHpFBxKfRzNkmi+
-3TBswZm2yqQ3qcrFN6H7ieUNCgN2Bj6tWlNmB/RCV+stbbEWwXbDlv1M+JNoM/Jw
-c1A++csAhx04BphNeKi01W0zEyRt6tlZWqolvhSUTom6xxWtjikqwfad6HMwn09V
-8luh5Ote2WNE8JcCggEBAMRQUlYKCTSvHuIscDukLExW+9JVKaufeXZvCuTK94Bs
-P0MtAgrHkgXdiRIBBFAViPjadMQitGkwKYd+zRPUmdzTuH9Lamqjl6e8dUpcO+Bf
-0scM94Te0gD8Gtv2Uu4Kw75P2RTXHRk9e8VbFpgYvfl9H0Uc8BK8jzlgWPqHgFnB
-WUXuMTAcKax4UwXdxWUjyv3xPmDygt7koPqpuX6FKvfxRq50a3TVIFku+8hRQtQR
-N+U0ScYrTOClTuc/QE0Q6rmw06lWHmytK3QiVhgpKxdw1xvVyjw5rMbB2eQdz7IQ
-2zn/o5+0gYdg/jvamo1HIxKjPCP1Lxm++manmy6kVD0CggEAeDwwm89yoJVEc6WZ
-uAClKFRjQDzbqNsU+ytBczcJsCSe8mpw0EWQOdhrDF4wxhZt9M6PTxqcGvd0R7pf
-8Hc2XCSNDGf/1/LGjTZ+I6wrVwJl1V8Kj+d3hH56ZscBDo3A5GDs7IH9acNtQ6Vw
-KkNVt48Bcmzk2/YiTelR/UXZMipoW5+bKUgGErcZONs8Nq6KCBIgjy1f2B/9cfIC
-4WhHkoFRr4aLwKdiwepf7BfFV5sSYxYtjuwCxF0UGln9PCjhBMuLlbZMmBSAFig1
-YhckBsgeNtVom+ULIaLBUkupYaWSOzs7SlmGx8eZGdr10CpTxYndEBiltjbGComx
-+ImsCQKCAQBaaX+yHock225Gzh6WaUL5man6sbwyTY0cLYH/4zZfz/rGzmi9XDJ1
-PxVM1GkPFQvzSHE0j6M1Org1rgF3G5gNKvkyryIAoP1MhDAkohv1d0xU3jT03cYs
-K++W1HhXJ2AFOzMINRYytK1XNF9QhzyfNa/8HZq3ll4EF8qC/3ruW2zpFw1SUfYj
-d3sNHZk2vmhT0hJfhfEeBH/bUeWbTmt+q4FZAUcoFKwERu4w0LQNhSyQBCfh+7k5
-UQjo2amclKj2AmlI+N+kP5DeuJ2cHQG6lv6K1EiCujFHjKn0NIKeSMMekAzklbZ+
-Cf6sxD4fyN5vS/x7twUNP3aFZrXCom4lAoIBABSGp/CGw6xeQDPXvZZ7gpLr5m5P
-mp+gS3S2MJZUv+6TEwXgHxnbxt5DMe25RAlqVHTuw9/TVccD6qcERvnhxUFMoscD
-5Vg/Hj+D1nPEVxvFgq0xEPI2FcwS6xT8gicV5dsCWIlOY3ZxBzOzByW1iA2YNymj
-pNTypqkARnheZZCK+4ortIeczll5XpbyGIXyxfIrKLYyv38gqLwCdb+MJsih91qc
-SrijfhjqqK9sr5KyHlPJDI4Mlw//qkPwIiKv70TGsSgu775OyyZFEl3uS+Y4BUGa
-dk5d+8OviBYIReMvvyl6U0jFCUK579lEDpEOPDF2C6VvDQ3FyHP7UXaOWrI=
------END RSA PRIVATE KEY-----

=== added file 'mandos'
--- mandos	1970-01-01 00:00:00 +0000
+++ mandos	2008-08-08 01:17:17 +0000
@@ -0,0 +1,827 @@
+#!/usr/bin/python
+# -*- mode: python; coding: utf-8 -*-
+# 
+# Mandos server - give out binary blobs to connecting clients.
+# 
+# This program is partly derived from an example program for an Avahi
+# service publisher, downloaded from
+# <http://avahi.org/wiki/PythonPublishExample>.  This includes the
+# methods "add" and "remove" in the "AvahiService" class, the
+# "server_state_changed" and "entry_group_state_changed" functions,
+# and some lines in "main".
+# 
+# Everything else is
+# Copyright © 2007-2008 Teddy Hogeborn & Björn Påhlsson
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+#     This program is distributed in the hope that it will be useful,
+#     but WITHOUT ANY WARRANTY; without even the implied warranty of
+#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#     GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# 
+# Contact the authors at <mandos@fukt.bsnet.se>.
+# 
+
+from __future__ import division
+
+import SocketServer
+import socket
+import select
+from optparse import OptionParser
+import datetime
+import errno
+import gnutls.crypto
+import gnutls.connection
+import gnutls.errors
+import gnutls.library.functions
+import gnutls.library.constants
+import gnutls.library.types
+import ConfigParser
+import sys
+import re
+import os
+import signal
+from sets import Set
+import subprocess
+import atexit
+import stat
+import logging
+import logging.handlers
+
+import dbus
+import gobject
+import avahi
+from dbus.mainloop.glib import DBusGMainLoop
+import ctypes
+
+
+logger = logging.Logger('mandos')
+syslogger = logging.handlers.SysLogHandler\
+            (facility = logging.handlers.SysLogHandler.LOG_DAEMON)
+syslogger.setFormatter(logging.Formatter\
+                        ('%(levelname)s: %(message)s'))
+logger.addHandler(syslogger)
+del syslogger
+
+
+class AvahiError(Exception):
+    def __init__(self, value):
+        self.value = value
+    def __str__(self):
+        return repr(self.value)
+
+class AvahiServiceError(AvahiError):
+    pass
+
+class AvahiGroupError(AvahiError):
+    pass
+
+
+class AvahiService(object):
+    """An Avahi (Zeroconf) service.
+    Attributes:
+    interface: integer; avahi.IF_UNSPEC or an interface index.
+               Used to optionally bind to the specified interface.
+    name: string; Example: 'Mandos'
+    type: string; Example: '_mandos._tcp'.
+                  See <http://www.dns-sd.org/ServiceTypes.html>
+    port: integer; what port to announce
+    TXT: list of strings; TXT record for the service
+    domain: string; Domain to publish on, default to .local if empty.
+    host: string; Host to publish records for, default is localhost
+    max_renames: integer; maximum number of renames
+    rename_count: integer; counter so we only rename after collisions
+                  a sensible number of times
+    """
+    def __init__(self, interface = avahi.IF_UNSPEC, name = None,
+                 type = None, port = None, TXT = None, domain = "",
+                 host = "", max_renames = 12):
+        self.interface = interface
+        self.name = name
+        self.type = type
+        self.port = port
+        if TXT is None:
+            self.TXT = []
+        else:
+            self.TXT = TXT
+        self.domain = domain
+        self.host = host
+        self.rename_count = 0
+    def rename(self):
+        """Derived from the Avahi example code"""
+        if self.rename_count >= self.max_renames:
+            logger.critical(u"No suitable service name found after %i"
+                            u" retries, exiting.", rename_count)
+            raise AvahiServiceError("Too many renames")
+        name = server.GetAlternativeServiceName(name)
+        logger.error(u"Changing name to %r ...", name)
+        self.remove()
+        self.add()
+        self.rename_count += 1
+    def remove(self):
+        """Derived from the Avahi example code"""
+        if group is not None:
+            group.Reset()
+    def add(self):
+        """Derived from the Avahi example code"""
+        global group
+        if group is None:
+            group = dbus.Interface\
+                    (bus.get_object(avahi.DBUS_NAME,
+                                    server.EntryGroupNew()),
+                     avahi.DBUS_INTERFACE_ENTRY_GROUP)
+            group.connect_to_signal('StateChanged',
+                                    entry_group_state_changed)
+        logger.debug(u"Adding service '%s' of type '%s' ...",
+                     service.name, service.type)
+        group.AddService(
+                self.interface,         # interface
+                avahi.PROTO_INET6,      # protocol
+                dbus.UInt32(0),         # flags
+                self.name, self.type,
+                self.domain, self.host,
+                dbus.UInt16(self.port),
+                avahi.string_array_to_txt_array(self.TXT))
+        group.Commit()
+
+# From the Avahi example code:
+group = None                            # our entry group
+# End of Avahi example code
+
+
+class Client(object):
+    """A representation of a client host served by this server.
+    Attributes:
+    name:      string; from the config file, used in log messages
+    fingerprint: string (40 or 32 hexadecimal digits); used to
+                 uniquely identify the client
+    secret:    bytestring; sent verbatim (over TLS) to client
+    fqdn:      string (FQDN); available for use by the checker command
+    created:   datetime.datetime(); object creation, not client host
+    last_checked_ok: datetime.datetime() or None if not yet checked OK
+    timeout:   datetime.timedelta(); How long from last_checked_ok
+                                     until this client is invalid
+    interval:  datetime.timedelta(); How often to start a new checker
+    stop_hook: If set, called by stop() as stop_hook(self)
+    checker:   subprocess.Popen(); a running checker process used
+                                   to see if the client lives.
+                                   'None' if no process is running.
+    checker_initiator_tag: a gobject event source tag, or None
+    stop_initiator_tag:    - '' -
+    checker_callback_tag:  - '' -
+    checker_command: string; External command which is run to check if
+                     client lives.  %() expansions are done at
+                     runtime with vars(self) as dict, so that for
+                     instance %(name)s can be used in the command.
+    Private attibutes:
+    _timeout: Real variable for 'timeout'
+    _interval: Real variable for 'interval'
+    _timeout_milliseconds: Used when calling gobject.timeout_add()
+    _interval_milliseconds: - '' -
+    """
+    def _set_timeout(self, timeout):
+        "Setter function for 'timeout' attribute"
+        self._timeout = timeout
+        self._timeout_milliseconds = ((self.timeout.days
+                                       * 24 * 60 * 60 * 1000)
+                                      + (self.timeout.seconds * 1000)
+                                      + (self.timeout.microseconds
+                                         // 1000))
+    timeout = property(lambda self: self._timeout,
+                       _set_timeout)
+    del _set_timeout
+    def _set_interval(self, interval):
+        "Setter function for 'interval' attribute"
+        self._interval = interval
+        self._interval_milliseconds = ((self.interval.days
+                                        * 24 * 60 * 60 * 1000)
+                                       + (self.interval.seconds
+                                          * 1000)
+                                       + (self.interval.microseconds
+                                          // 1000))
+    interval = property(lambda self: self._interval,
+                        _set_interval)
+    del _set_interval
+    def __init__(self, name = None, stop_hook=None, config={}):
+        """Note: the 'checker' key in 'config' sets the
+        'checker_command' attribute and *not* the 'checker'
+        attribute."""
+        self.name = name
+        logger.debug(u"Creating client %r", self.name)
+        # Uppercase and remove spaces from fingerprint for later
+        # comparison purposes with return value from the fingerprint()
+        # function
+        self.fingerprint = config["fingerprint"].upper()\
+                           .replace(u" ", u"")
+        logger.debug(u"  Fingerprint: %s", self.fingerprint)
+        if "secret" in config:
+            self.secret = config["secret"].decode(u"base64")
+        elif "secfile" in config:
+            sf = open(config["secfile"])
+            self.secret = sf.read()
+            sf.close()
+        else:
+            raise TypeError(u"No secret or secfile for client %s"
+                            % self.name)
+        self.fqdn = config.get("fqdn", "")
+        self.created = datetime.datetime.now()
+        self.last_checked_ok = None
+        self.timeout = string_to_delta(config["timeout"])
+        self.interval = string_to_delta(config["interval"])
+        self.stop_hook = stop_hook
+        self.checker = None
+        self.checker_initiator_tag = None
+        self.stop_initiator_tag = None
+        self.checker_callback_tag = None
+        self.check_command = config["checker"]
+    def start(self):
+        """Start this client's checker and timeout hooks"""
+        # Schedule a new checker to be started an 'interval' from now,
+        # and every interval from then on.
+        self.checker_initiator_tag = gobject.timeout_add\
+                                     (self._interval_milliseconds,
+                                      self.start_checker)
+        # Also start a new checker *right now*.
+        self.start_checker()
+        # Schedule a stop() when 'timeout' has passed
+        self.stop_initiator_tag = gobject.timeout_add\
+                                  (self._timeout_milliseconds,
+                                   self.stop)
+    def stop(self):
+        """Stop this client.
+        The possibility that a client might be restarted is left open,
+        but not currently used."""
+        # If this client doesn't have a secret, it is already stopped.
+        if self.secret:
+            logger.info(u"Stopping client %s", self.name)
+            self.secret = None
+        else:
+            return False
+        if getattr(self, "stop_initiator_tag", False):
+            gobject.source_remove(self.stop_initiator_tag)
+            self.stop_initiator_tag = None
+        if getattr(self, "checker_initiator_tag", False):
+            gobject.source_remove(self.checker_initiator_tag)
+            self.checker_initiator_tag = None
+        self.stop_checker()
+        if self.stop_hook:
+            self.stop_hook(self)
+        # Do not run this again if called by a gobject.timeout_add
+        return False
+    def __del__(self):
+        self.stop_hook = None
+        self.stop()
+    def checker_callback(self, pid, condition):
+        """The checker has completed, so take appropriate actions."""
+        now = datetime.datetime.now()
+        self.checker_callback_tag = None
+        self.checker = None
+        if os.WIFEXITED(condition) \
+               and (os.WEXITSTATUS(condition) == 0):
+            logger.info(u"Checker for %(name)s succeeded",
+                        vars(self))
+            self.last_checked_ok = now
+            gobject.source_remove(self.stop_initiator_tag)
+            self.stop_initiator_tag = gobject.timeout_add\
+                                      (self._timeout_milliseconds,
+                                       self.stop)
+        elif not os.WIFEXITED(condition):
+            logger.warning(u"Checker for %(name)s crashed?",
+                           vars(self))
+        else:
+            logger.info(u"Checker for %(name)s failed",
+                        vars(self))
+    def start_checker(self):
+        """Start a new checker subprocess if one is not running.
+        If a checker already exists, leave it running and do
+        nothing."""
+        # The reason for not killing a running checker is that if we
+        # did that, then if a checker (for some reason) started
+        # running slowly and taking more than 'interval' time, the
+        # client would inevitably timeout, since no checker would get
+        # a chance to run to completion.  If we instead leave running
+        # checkers alone, the checker would have to take more time
+        # than 'timeout' for the client to be declared invalid, which
+        # is as it should be.
+        if self.checker is None:
+            try:
+                # In case check_command has exactly one % operator
+                command = self.check_command % self.fqdn
+            except TypeError:
+                # Escape attributes for the shell
+                escaped_attrs = dict((key, re.escape(str(val)))
+                                     for key, val in
+                                     vars(self).iteritems())
+                try:
+                    command = self.check_command % escaped_attrs
+                except TypeError, error:
+                    logger.error(u'Could not format string "%s":'
+                                 u' %s', self.check_command, error)
+                    return True # Try again later
+            try:
+                logger.info(u"Starting checker %r for %s",
+                            command, self.name)
+                self.checker = subprocess.Popen(command,
+                                                close_fds=True,
+                                                shell=True, cwd="/")
+                self.checker_callback_tag = gobject.child_watch_add\
+                                            (self.checker.pid,
+                                             self.checker_callback)
+            except subprocess.OSError, error:
+                logger.error(u"Failed to start subprocess: %s",
+                             error)
+        # Re-run this periodically if run by gobject.timeout_add
+        return True
+    def stop_checker(self):
+        """Force the checker process, if any, to stop."""
+        if self.checker_callback_tag:
+            gobject.source_remove(self.checker_callback_tag)
+            self.checker_callback_tag = None
+        if getattr(self, "checker", None) is None:
+            return
+        logger.debug("Stopping checker for %(name)s", vars(self))
+        try:
+            os.kill(self.checker.pid, signal.SIGTERM)
+            #os.sleep(0.5)
+            #if self.checker.poll() is None:
+            #    os.kill(self.checker.pid, signal.SIGKILL)
+        except OSError, error:
+            if error.errno != errno.ESRCH: # No such process
+                raise
+        self.checker = None
+    def still_valid(self):
+        """Has the timeout not yet passed for this client?"""
+        now = datetime.datetime.now()
+        if self.last_checked_ok is None:
+            return now < (self.created + self.timeout)
+        else:
+            return now < (self.last_checked_ok + self.timeout)
+
+
+def peer_certificate(session):
+    "Return the peer's OpenPGP certificate as a bytestring"
+    # If not an OpenPGP certificate...
+    if gnutls.library.functions.gnutls_certificate_type_get\
+            (session._c_object) \
+           != gnutls.library.constants.GNUTLS_CRT_OPENPGP:
+        # ...do the normal thing
+        return session.peer_certificate
+    list_size = ctypes.c_uint()
+    cert_list = gnutls.library.functions.gnutls_certificate_get_peers\
+        (session._c_object, ctypes.byref(list_size))
+    if list_size.value == 0:
+        return None
+    cert = cert_list[0]
+    return ctypes.string_at(cert.data, cert.size)
+
+
+def fingerprint(openpgp):
+    "Convert an OpenPGP bytestring to a hexdigit fingerprint string"
+    # New GnuTLS "datum" with the OpenPGP public key
+    datum = gnutls.library.types.gnutls_datum_t\
+        (ctypes.cast(ctypes.c_char_p(openpgp),
+                     ctypes.POINTER(ctypes.c_ubyte)),
+         ctypes.c_uint(len(openpgp)))
+    # New empty GnuTLS certificate
+    crt = gnutls.library.types.gnutls_openpgp_crt_t()
+    gnutls.library.functions.gnutls_openpgp_crt_init\
+        (ctypes.byref(crt))
+    # Import the OpenPGP public key into the certificate
+    gnutls.library.functions.gnutls_openpgp_crt_import\
+                    (crt, ctypes.byref(datum),
+                     gnutls.library.constants.GNUTLS_OPENPGP_FMT_RAW)
+    # New buffer for the fingerprint
+    buffer = ctypes.create_string_buffer(20)
+    buffer_length = ctypes.c_size_t()
+    # Get the fingerprint from the certificate into the buffer
+    gnutls.library.functions.gnutls_openpgp_crt_get_fingerprint\
+        (crt, ctypes.byref(buffer), ctypes.byref(buffer_length))
+    # Deinit the certificate
+    gnutls.library.functions.gnutls_openpgp_crt_deinit(crt)
+    # Convert the buffer to a Python bytestring
+    fpr = ctypes.string_at(buffer, buffer_length.value)
+    # Convert the bytestring to hexadecimal notation
+    hex_fpr = u''.join(u"%02X" % ord(char) for char in fpr)
+    return hex_fpr
+
+
+class tcp_handler(SocketServer.BaseRequestHandler, object):
+    """A TCP request handler class.
+    Instantiated by IPv6_TCPServer for each request to handle it.
+    Note: This will run in its own forked process."""
+    
+    def handle(self):
+        logger.info(u"TCP connection from: %s",
+                     unicode(self.client_address))
+        session = gnutls.connection.ClientSession\
+                  (self.request, gnutls.connection.X509Credentials())
+        
+        line = self.request.makefile().readline()
+        logger.debug(u"Protocol version: %r", line)
+        try:
+            if int(line.strip().split()[0]) > 1:
+                raise RuntimeError
+        except (ValueError, IndexError, RuntimeError), error:
+            logger.error(u"Unknown protocol version: %s", error)
+            return
+        
+        # Note: gnutls.connection.X509Credentials is really a generic
+        # GnuTLS certificate credentials object so long as no X.509
+        # keys are added to it.  Therefore, we can use it here despite
+        # using OpenPGP certificates.
+        
+        #priority = ':'.join(("NONE", "+VERS-TLS1.1", "+AES-256-CBC",
+        #                "+SHA1", "+COMP-NULL", "+CTYPE-OPENPGP",
+        #                "+DHE-DSS"))
+        priority = "NORMAL"             # Fallback default, since this
+                                        # MUST be set.
+        if self.server.settings["priority"]:
+            priority = self.server.settings["priority"]
+        gnutls.library.functions.gnutls_priority_set_direct\
+            (session._c_object, priority, None);
+        
+        try:
+            session.handshake()
+        except gnutls.errors.GNUTLSError, error:
+            logger.warning(u"Handshake failed: %s", error)
+            # Do not run session.bye() here: the session is not
+            # established.  Just abandon the request.
+            return
+        try:
+            fpr = fingerprint(peer_certificate(session))
+        except (TypeError, gnutls.errors.GNUTLSError), error:
+            logger.warning(u"Bad certificate: %s", error)
+            session.bye()
+            return
+        logger.debug(u"Fingerprint: %s", fpr)
+        client = None
+        for c in self.server.clients:
+            if c.fingerprint == fpr:
+                client = c
+                break
+        if not client:
+            logger.warning(u"Client not found for fingerprint: %s",
+                           fpr)
+            session.bye()
+            return
+        # Have to check if client.still_valid(), since it is possible
+        # that the client timed out while establishing the GnuTLS
+        # session.
+        if not client.still_valid():
+            logger.warning(u"Client %(name)s is invalid",
+                           vars(client))
+            session.bye()
+            return
+        sent_size = 0
+        while sent_size < len(client.secret):
+            sent = session.send(client.secret[sent_size:])
+            logger.debug(u"Sent: %d, remaining: %d",
+                         sent, len(client.secret)
+                         - (sent_size + sent))
+            sent_size += sent
+        session.bye()
+
+
+class IPv6_TCPServer(SocketServer.ForkingTCPServer, object):
+    """IPv6 TCP server.  Accepts 'None' as address and/or port.
+    Attributes:
+        settings:       Server settings
+        clients:        Set() of Client objects
+    """
+    address_family = socket.AF_INET6
+    def __init__(self, *args, **kwargs):
+        if "settings" in kwargs:
+            self.settings = kwargs["settings"]
+            del kwargs["settings"]
+        if "clients" in kwargs:
+            self.clients = kwargs["clients"]
+            del kwargs["clients"]
+        return super(type(self), self).__init__(*args, **kwargs)
+    def server_bind(self):
+        """This overrides the normal server_bind() function
+        to bind to an interface if one was specified, and also NOT to
+        bind to an address or port if they were not specified."""
+        if self.settings["interface"]:
+            # 25 is from /usr/include/asm-i486/socket.h
+            SO_BINDTODEVICE = getattr(socket, "SO_BINDTODEVICE", 25)
+            try:
+                self.socket.setsockopt(socket.SOL_SOCKET,
+                                       SO_BINDTODEVICE,
+                                       self.settings["interface"])
+            except socket.error, error:
+                if error[0] == errno.EPERM:
+                    logger.error(u"No permission to"
+                                 u" bind to interface %s",
+                                 self.settings["interface"])
+                else:
+                    raise error
+        # Only bind(2) the socket if we really need to.
+        if self.server_address[0] or self.server_address[1]:
+            if not self.server_address[0]:
+                in6addr_any = "::"
+                self.server_address = (in6addr_any,
+                                       self.server_address[1])
+            elif not self.server_address[1]:
+                self.server_address = (self.server_address[0],
+                                       0)
+#                 if self.settings["interface"]:
+#                     self.server_address = (self.server_address[0],
+#                                            0, # port
+#                                            0, # flowinfo
+#                                            if_nametoindex
+#                                            (self.settings
+#                                             ["interface"]))
+            return super(type(self), self).server_bind()
+
+
+def string_to_delta(interval):
+    """Parse a string and return a datetime.timedelta
+
+    >>> string_to_delta('7d')
+    datetime.timedelta(7)
+    >>> string_to_delta('60s')
+    datetime.timedelta(0, 60)
+    >>> string_to_delta('60m')
+    datetime.timedelta(0, 3600)
+    >>> string_to_delta('24h')
+    datetime.timedelta(1)
+    >>> string_to_delta(u'1w')
+    datetime.timedelta(7)
+    """
+    try:
+        suffix=unicode(interval[-1])
+        value=int(interval[:-1])
+        if suffix == u"d":
+            delta = datetime.timedelta(value)
+        elif suffix == u"s":
+            delta = datetime.timedelta(0, value)
+        elif suffix == u"m":
+            delta = datetime.timedelta(0, 0, 0, 0, value)
+        elif suffix == u"h":
+            delta = datetime.timedelta(0, 0, 0, 0, 0, value)
+        elif suffix == u"w":
+            delta = datetime.timedelta(0, 0, 0, 0, 0, 0, value)
+        else:
+            raise ValueError
+    except (ValueError, IndexError):
+        raise ValueError
+    return delta
+
+
+def server_state_changed(state):
+    """Derived from the Avahi example code"""
+    if state == avahi.SERVER_COLLISION:
+        logger.error(u"Server name collision")
+        service.remove()
+    elif state == avahi.SERVER_RUNNING:
+        service.add()
+
+
+def entry_group_state_changed(state, error):
+    """Derived from the Avahi example code"""
+    logger.debug(u"state change: %i", state)
+    
+    if state == avahi.ENTRY_GROUP_ESTABLISHED:
+        logger.debug(u"Service established.")
+    elif state == avahi.ENTRY_GROUP_COLLISION:
+        logger.warning(u"Service name collision.")
+        service.rename()
+    elif state == avahi.ENTRY_GROUP_FAILURE:
+        logger.critical(u"Error in group state changed %s",
+                        unicode(error))
+        raise AvahiGroupError("State changed: %s", str(error))
+
+def if_nametoindex(interface):
+    """Call the C function if_nametoindex(), or equivalent"""
+    global if_nametoindex
+    try:
+        if "ctypes.util" not in sys.modules:
+            import ctypes.util
+        if_nametoindex = ctypes.cdll.LoadLibrary\
+            (ctypes.util.find_library("c")).if_nametoindex
+    except (OSError, AttributeError):
+        if "struct" not in sys.modules:
+            import struct
+        if "fcntl" not in sys.modules:
+            import fcntl
+        def if_nametoindex(interface):
+            "Get an interface index the hard way, i.e. using fcntl()"
+            SIOCGIFINDEX = 0x8933  # From /usr/include/linux/sockios.h
+            s = socket.socket()
+            ifreq = fcntl.ioctl(s, SIOCGIFINDEX,
+                                struct.pack("16s16x", interface))
+            s.close()
+            interface_index = struct.unpack("I", ifreq[16:20])[0]
+            return interface_index
+    return if_nametoindex(interface)
+
+
+def daemon(nochdir = False, noclose = False):
+    """See daemon(3).  Standard BSD Unix function.
+    This should really exist as os.daemon, but it doesn't (yet)."""
+    if os.fork():
+        sys.exit()
+    os.setsid()
+    if not nochdir:
+        os.chdir("/")
+    if os.fork():
+        sys.exit()
+    if not noclose:
+        # Close all standard open file descriptors
+        null = os.open(os.path.devnull, os.O_NOCTTY | os.O_RDWR)
+        if not stat.S_ISCHR(os.fstat(null).st_mode):
+            raise OSError(errno.ENODEV,
+                          "/dev/null not a character device")
+        os.dup2(null, sys.stdin.fileno())
+        os.dup2(null, sys.stdout.fileno())
+        os.dup2(null, sys.stderr.fileno())
+        if null > 2:
+            os.close(null)
+
+
+def main():
+    global main_loop_started
+    main_loop_started = False
+    
+    parser = OptionParser()
+    parser.add_option("-i", "--interface", type="string",
+                      metavar="IF", help="Bind to interface IF")
+    parser.add_option("-a", "--address", type="string",
+                      help="Address to listen for requests on")
+    parser.add_option("-p", "--port", type="int",
+                      help="Port number to receive requests on")
+    parser.add_option("--check", action="store_true", default=False,
+                      help="Run self-test")
+    parser.add_option("--debug", action="store_true",
+                      help="Debug mode; run in foreground and log to"
+                      " terminal")
+    parser.add_option("--priority", type="string", help="GnuTLS"
+                      " priority string (see GnuTLS documentation)")
+    parser.add_option("--servicename", type="string", metavar="NAME",
+                      help="Zeroconf service name")
+    parser.add_option("--configdir", type="string",
+                      default="/etc/mandos", metavar="DIR",
+                      help="Directory to search for configuration"
+                      " files")
+    (options, args) = parser.parse_args()
+    
+    if options.check:
+        import doctest
+        doctest.testmod()
+        sys.exit()
+    
+    # Default values for config file for server-global settings
+    server_defaults = { "interface": "",
+                        "address": "",
+                        "port": "",
+                        "debug": "False",
+                        "priority":
+                        "SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP",
+                        "servicename": "Mandos",
+                        }
+    
+    # Parse config file for server-global settings
+    server_config = ConfigParser.SafeConfigParser(server_defaults)
+    del server_defaults
+    server_config.read(os.path.join(options.configdir, "mandos.conf"))
+    server_section = "server"
+    # Convert the SafeConfigParser object to a dict
+    server_settings = dict(server_config.items(server_section))
+    # Use getboolean on the boolean config option
+    server_settings["debug"] = server_config.getboolean\
+                               (server_section, "debug")
+    del server_config
+    
+    # Override the settings from the config file with command line
+    # options, if set.
+    for option in ("interface", "address", "port", "debug",
+                   "priority", "servicename", "configdir"):
+        value = getattr(options, option)
+        if value is not None:
+            server_settings[option] = value
+    del options
+    # Now we have our good server settings in "server_settings"
+    
+    # Parse config file with clients
+    client_defaults = { "timeout": "1h",
+                        "interval": "5m",
+                        "checker": "fping -q -- %%(fqdn)s",
+                        }
+    client_config = ConfigParser.SafeConfigParser(client_defaults)
+    client_config.read(os.path.join(server_settings["configdir"],
+                                    "clients.conf"))
+    
+    global service
+    service = AvahiService(name = server_settings["servicename"],
+                           type = "_mandos._tcp", );
+    if server_settings["interface"]:
+        service.interface = if_nametoindex(server_settings["interface"])
+    
+    global main_loop
+    global bus
+    global server
+    # From the Avahi example code
+    DBusGMainLoop(set_as_default=True )
+    main_loop = gobject.MainLoop()
+    bus = dbus.SystemBus()
+    server = dbus.Interface(
+            bus.get_object( avahi.DBUS_NAME, avahi.DBUS_PATH_SERVER ),
+            avahi.DBUS_INTERFACE_SERVER )
+    # End of Avahi example code
+    
+    debug = server_settings["debug"]
+    
+    if debug:
+        console = logging.StreamHandler()
+        # console.setLevel(logging.DEBUG)
+        console.setFormatter(logging.Formatter\
+                             ('%(levelname)s: %(message)s'))
+        logger.addHandler(console)
+        del console
+    
+    clients = Set()
+    def remove_from_clients(client):
+        clients.remove(client)
+        if not clients:
+            logger.critical(u"No clients left, exiting")
+            sys.exit()
+    
+    clients.update(Set(Client(name = section,
+                              stop_hook = remove_from_clients,
+                              config
+                              = dict(client_config.items(section)))
+                       for section in client_config.sections()))
+    
+    if not debug:
+        daemon()
+    
+    def cleanup():
+        "Cleanup function; run on exit"
+        global group
+        # From the Avahi example code
+        if not group is None:
+            group.Free()
+            group = None
+        # End of Avahi example code
+        
+        while clients:
+            client = clients.pop()
+            client.stop_hook = None
+            client.stop()
+    
+    atexit.register(cleanup)
+    
+    if not debug:
+        signal.signal(signal.SIGINT, signal.SIG_IGN)
+    signal.signal(signal.SIGHUP, lambda signum, frame: sys.exit())
+    signal.signal(signal.SIGTERM, lambda signum, frame: sys.exit())
+    
+    for client in clients:
+        client.start()
+    
+    tcp_server = IPv6_TCPServer((server_settings["address"],
+                                 server_settings["port"]),
+                                tcp_handler,
+                                settings=server_settings,
+                                clients=clients)
+    # Find out what port we got
+    service.port = tcp_server.socket.getsockname()[1]
+    logger.info(u"Now listening on address %r, port %d, flowinfo %d,"
+                u" scope_id %d" % tcp_server.socket.getsockname())
+    
+    #service.interface = tcp_server.socket.getsockname()[3]
+    
+    try:
+        # From the Avahi example code
+        server.connect_to_signal("StateChanged", server_state_changed)
+        try:
+            server_state_changed(server.GetState())
+        except dbus.exceptions.DBusException, error:
+            logger.critical(u"DBusException: %s", error)
+            sys.exit(1)
+        # End of Avahi example code
+        
+        gobject.io_add_watch(tcp_server.fileno(), gobject.IO_IN,
+                             lambda *args, **kwargs:
+                             tcp_server.handle_request\
+                             (*args[2:], **kwargs) or True)
+        
+        logger.debug("Starting main loop")
+        main_loop_started = True
+        main_loop.run()
+    except AvahiError, error:
+        logger.critical(u"AvahiError: %s" + unicode(error))
+        sys.exit(1)
+    except KeyboardInterrupt:
+        if debug:
+            print
+
+if __name__ == '__main__':
+    main()

=== added file 'mandos-client.c'
--- mandos-client.c	1970-01-01 00:00:00 +0000
+++ mandos-client.c	2008-08-09 04:42:39 +0000
@@ -0,0 +1,743 @@
+/*  -*- coding: utf-8 -*- */
+/*
+ * Mandos plugin runner - Run Mandos plugins
+ *
+ * Copyright © 2007-2008 Teddy Hogeborn & Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ * 
+ * Contact the authors at <mandos@fukt.bsnet.se>.
+ */
+
+#define _GNU_SOURCE		/* TEMP_FAILURE_RETRY() */
+
+#include <stddef.h>		/* size_t, NULL */
+#include <stdlib.h>		/* malloc(), exit(), EXIT_FAILURE,
+				   EXIT_SUCCESS, realloc() */
+#include <stdbool.h>		/* bool, true, false */
+#include <stdio.h>		/* perror, popen(), fileno(),
+				   fprintf(), stderr, STDOUT_FILENO */
+#include <sys/types.h>	        /* DIR, opendir(), stat(), struct
+				   stat, waitpid(), WIFEXITED(),
+				   WEXITSTATUS(), wait(), pid_t,
+				   uid_t, gid_t, getuid(), getgid(),
+				   dirfd() */
+#include <sys/select.h>		/* fd_set, select(), FD_ZERO(),
+				   FD_SET(), FD_ISSET(), FD_CLR */
+#include <sys/wait.h>		/* wait(), waitpid(), WIFEXITED(),
+				   WEXITSTATUS() */
+#include <sys/stat.h>		/* struct stat, stat(), S_ISREG() */
+#include <iso646.h>		/* and, or, not */
+#include <dirent.h>		/* DIR, struct dirent, opendir(),
+				   readdir(), closedir(), dirfd() */
+#include <unistd.h>		/* struct stat, stat(), S_ISREG(),
+				   fcntl(), setuid(), setgid(),
+				   F_GETFD, F_SETFD, FD_CLOEXEC,
+				   access(), pipe(), fork(), close()
+				   dup2, STDOUT_FILENO, _exit(),
+				   execv(), write(), read(),
+				   close() */
+#include <fcntl.h>		/* fcntl(), F_GETFD, F_SETFD,
+				   FD_CLOEXEC */
+#include <string.h>		/* strtok, strlen(), strcpy(),
+				   strcat() */
+#include <errno.h>		/* errno */
+#include <argp.h>		/* struct argp_option, struct
+				   argp_state, struct argp,
+				   argp_parse(), ARGP_ERR_UNKNOWN,
+				   ARGP_KEY_END, ARGP_KEY_ARG, error_t */
+#include <signal.h> 		/* struct sigaction, sigemptyset(),
+				   sigaddset(), sigaction(),
+				   sigprocmask(), SIG_BLOCK, SIGCHLD,
+				   SIG_UNBLOCK, kill() */
+#include <errno.h>		/* errno, EBADF */
+
+#define BUFFER_SIZE 256
+
+struct process;
+
+typedef struct process{
+  pid_t pid;
+  int fd;
+  char *buffer;
+  size_t buffer_size;
+  size_t buffer_length;
+  bool eof;
+  bool completed;
+  int status;
+  struct process *next;
+} process;
+
+typedef struct plugin{
+  char *name;			/* can be NULL or any plugin name */
+  char **argv;
+  int argc;
+  bool disabled;
+  struct plugin *next;
+} plugin;
+
+static plugin *getplugin(char *name, plugin **plugin_list){
+  for (plugin *p = *plugin_list; p != NULL; p = p->next){
+    if ((p->name == name)
+	or (p->name and name and (strcmp(p->name, name) == 0))){
+      return p;
+    }
+  }
+  /* Create a new plugin */
+  plugin *new_plugin = malloc(sizeof(plugin));
+  if (new_plugin == NULL){
+    perror("malloc");
+    exit(EXIT_FAILURE);
+  }
+  new_plugin->name = name;
+  new_plugin->argv = malloc(sizeof(char *) * 2);
+  if (new_plugin->argv == NULL){
+    perror("malloc");
+    exit(EXIT_FAILURE);
+  }
+  new_plugin->argv[0] = name;
+  new_plugin->argv[1] = NULL;
+  new_plugin->argc = 1;
+  new_plugin->disabled = false;
+  new_plugin->next = *plugin_list;
+  /* Append the new plugin to the list */
+  *plugin_list = new_plugin;
+  return new_plugin;
+}
+
+static void addargument(plugin *p, char *arg){
+  p->argv[p->argc] = arg;
+  p->argv = realloc(p->argv, sizeof(char *) * (size_t)(p->argc + 2));
+  if (p->argv == NULL){
+    perror("malloc");
+    exit(EXIT_FAILURE);
+  }
+  p->argc++;
+  p->argv[p->argc] = NULL;
+}
+
+/*
+ * Based on the example in the GNU LibC manual chapter 13.13 "File
+ * Descriptor Flags".
+ * *Note File Descriptor Flags:(libc)Descriptor Flags.
+ */
+static int set_cloexec_flag(int fd)
+{
+  int ret = fcntl(fd, F_GETFD, 0);
+  /* If reading the flags failed, return error indication now. */
+  if(ret < 0){
+    return ret;
+  }
+  /* Store modified flag word in the descriptor. */
+  return fcntl(fd, F_SETFD, ret | FD_CLOEXEC);
+}
+
+const char *argp_program_version = "plugbasedclient 0.9";
+const char *argp_program_bug_address = "<mandos@fukt.bsnet.se>";
+
+process *process_list = NULL;
+
+/* Mark a process as completed when it exits, and save its exit
+   status. */
+void handle_sigchld(__attribute__((unused)) int sig){
+  process *proc = process_list;
+  int status;
+  pid_t pid = wait(&status);
+  if(pid == -1){
+    perror("wait");
+    return;
+  }
+  while(proc != NULL and proc->pid != pid){
+    proc = proc->next;
+  }
+  if(proc == NULL){
+    /* Process not found in process list */
+    return;
+  }
+  proc->status = status;
+  proc->completed = true;
+}
+
+int main(int argc, char *argv[]){
+  const char *plugindir = "/conf/conf.d/mandos/plugins.d";
+  size_t d_name_len;
+  DIR *dir = NULL;
+  struct dirent *dirst;
+  struct stat st;
+  fd_set rfds_all;
+  int ret, maxfd = 0;
+  uid_t uid = 65534;
+  gid_t gid = 65534;
+  bool debug = false;
+  int exitstatus = EXIT_SUCCESS;
+  struct sigaction old_sigchld_action;
+  struct sigaction sigchld_action = { .sa_handler = handle_sigchld,
+				      .sa_flags = SA_NOCLDSTOP };
+  char *plus_options = NULL;
+  char **plus_argv = NULL;
+  
+  /* Establish a signal handler */
+  sigemptyset(&sigchld_action.sa_mask);
+  ret = sigaddset(&sigchld_action.sa_mask, SIGCHLD);
+  if(ret < 0){
+    perror("sigaddset");
+    exit(EXIT_FAILURE);
+  }
+  ret = sigaction(SIGCHLD, &sigchld_action, &old_sigchld_action);
+  if(ret < 0){
+    perror("sigaction");
+    exit(EXIT_FAILURE);
+  }
+  
+  /* The options we understand. */
+  struct argp_option options[] = {
+    { .name = "global-options", .key = 'g',
+      .arg = "OPTION[,OPTION[,...]]",
+      .doc = "Options passed to all plugins" },
+    { .name = "options-for", .key = 'o',
+      .arg = "PLUGIN:OPTION[,OPTION[,...]]",
+      .doc = "Options passed only to specified plugin" },
+    { .name = "disable", .key = 'd',
+      .arg = "PLUGIN",
+      .doc = "Disable a specific plugin", .group = 1 },
+    { .name = "plugin-dir", .key = 128,
+      .arg = "DIRECTORY",
+      .doc = "Specify a different plugin directory", .group = 2 },
+    { .name = "userid", .key = 129,
+      .arg = "ID", .flags = 0,
+      .doc = "User ID the plugins will run as", .group = 2 },
+    { .name = "groupid", .key = 130,
+      .arg = "ID", .flags = 0,
+      .doc = "Group ID the plugins will run as", .group = 2 },
+    { .name = "debug", .key = 131,
+      .doc = "Debug mode", .group = 3 },
+    { .name = NULL }
+  };
+  
+  error_t parse_opt (int key, char *arg, struct argp_state *state) {
+    /* Get the INPUT argument from `argp_parse', which we know is a
+       pointer to our plugin list pointer. */
+    plugin **plugins = state->input;
+    switch (key) {
+    case 'g':
+      if (arg != NULL){
+	char *p = strtok(arg, ",");
+	do{
+	  addargument(getplugin(NULL, plugins), p);
+	  p = strtok(NULL, ",");
+	} while (p != NULL);
+      }
+      break;
+    case 'o':
+      if (arg != NULL){
+	char *name = strtok(arg, ":");
+	char *p = strtok(NULL, ":");
+	if(p != NULL){
+	  p = strtok(p, ",");
+	  do{
+	    addargument(getplugin(name, plugins), p);
+	    p = strtok(NULL, ",");
+	  } while (p != NULL);
+	}
+      }
+      break;
+    case 'd':
+      if (arg != NULL){
+	getplugin(arg, plugins)->disabled = true;
+      }
+      break;
+    case 128:
+      plugindir = arg;
+      break;
+    case 129:
+      uid = (uid_t)strtol(arg, NULL, 10);
+      break;
+    case 130:
+      gid = (gid_t)strtol(arg, NULL, 10);
+      break;
+    case 131:
+      debug = true;
+      break;
+    case ARGP_KEY_ARG:
+      if(plus_options != NULL or arg == NULL or arg[0] != '+'){
+	argp_usage (state);
+      }
+      plus_options = arg;
+      break;
+    case ARGP_KEY_END:
+      break;
+    default:
+      return ARGP_ERR_UNKNOWN;
+    }
+    return 0;
+  }
+  
+  plugin *plugin_list = NULL;
+  
+  struct argp argp = { .options = options, .parser = parse_opt,
+		       .args_doc = "[+PLUS_SEPARATED_OPTIONS]",
+		       .doc = "Mandos plugin runner -- Run plugins" };
+  
+  ret = argp_parse (&argp, argc, argv, 0, 0, &plugin_list);
+  if (ret == ARGP_ERR_UNKNOWN){
+    perror("argp_parse");
+    exitstatus = EXIT_FAILURE;
+    goto end;
+  }
+  
+  if(plus_options){
+    /* This is a mangled argument in the form of
+     "+--option+--other-option=parameter+--yet-another-option", etc */
+    /* Make new argc and argv vars, and call argp_parse() again. */
+    plus_options++;		/* skip the first '+' character */
+    const char delims[] = "+";
+    char *arg;
+    int new_argc = 1;
+    plus_argv = malloc(sizeof(char*) * 2);
+    if(plus_argv == NULL){
+      perror("malloc");
+      exitstatus = EXIT_FAILURE;
+      goto end;
+    }
+    plus_argv[0] = argv[0];
+    plus_argv[1] = NULL;
+    arg = strtok(plus_options, delims); /* Get first argument */
+    while(arg != NULL){
+      new_argc++;
+      plus_argv = realloc(plus_argv, sizeof(char *)
+			 * ((unsigned int) new_argc + 1));
+      if(plus_argv == NULL){
+	perror("realloc");
+	exitstatus = EXIT_FAILURE;
+	goto end;
+      }
+      plus_argv[new_argc-1] = arg;
+      plus_argv[new_argc] = NULL;
+      arg = strtok(NULL, delims); /* Get next argument */
+    }
+    ret = argp_parse (&argp, new_argc, plus_argv, 0, 0, &plugin_list);
+    if (ret == ARGP_ERR_UNKNOWN){
+      perror("argp_parse");
+      exitstatus = EXIT_FAILURE;
+      goto end;
+    }
+  }
+  
+  if(debug){
+    for(plugin *p = plugin_list; p != NULL; p=p->next){
+      fprintf(stderr, "Plugin: %s has %d arguments\n",
+	      p->name ? p->name : "Global", p->argc - 1);
+      for(char **a = p->argv; *a != NULL; a++){
+	fprintf(stderr, "\tArg: %s\n", *a);
+      }
+    }
+  }
+  
+  ret = setuid(uid);
+  if (ret == -1){
+    perror("setuid");
+  }
+  
+  setgid(gid);
+  if (ret == -1){
+    perror("setgid");
+  }
+  
+  dir = opendir(plugindir);
+  if(dir == NULL){
+    perror("Could not open plugin dir");
+    exitstatus = EXIT_FAILURE;
+    goto end;
+  }
+  
+  /* Set the FD_CLOEXEC flag on the directory, if possible */
+  {
+    int dir_fd = dirfd(dir);
+    if(dir_fd >= 0){
+      ret = set_cloexec_flag(dir_fd);
+      if(ret < 0){
+	perror("set_cloexec_flag");
+	exitstatus = EXIT_FAILURE;
+	goto end;
+      }
+    }
+  }
+  
+  FD_ZERO(&rfds_all);
+  
+  while(true){
+    dirst = readdir(dir);
+    
+    // All directory entries have been processed
+    if(dirst == NULL){
+      if (errno == EBADF){
+	perror("readdir");
+	exitstatus = EXIT_FAILURE;
+	goto end;
+      }
+      break;
+    }
+    
+    d_name_len = strlen(dirst->d_name);
+    
+    // Ignore dotfiles, backup files and other junk
+    {
+      bool bad_name = false;
+      
+      const char const *bad_prefixes[] = { ".", "#", NULL };
+      
+      const char const *bad_suffixes[] = { "~", "#", ".dpkg-new",
+					   ".dpkg-old",
+					   ".dpkg-divert", NULL };
+      for(const char **pre = bad_prefixes; *pre != NULL; pre++){
+	size_t pre_len = strlen(*pre);
+	if((d_name_len >= pre_len)
+	   and strncmp((dirst->d_name), *pre, pre_len) == 0){
+	  if(debug){
+	    fprintf(stderr, "Ignoring plugin dir entry \"%s\""
+		    " with bad prefix %s\n", dirst->d_name, *pre);
+	  }
+	  bad_name = true;
+	  break;
+	}
+      }
+      
+      if(bad_name){
+	continue;
+      }
+      
+      for(const char **suf = bad_suffixes; *suf != NULL; suf++){
+	size_t suf_len = strlen(*suf);
+	if((d_name_len >= suf_len)
+	   and (strcmp((dirst->d_name)+d_name_len-suf_len, *suf)
+		== 0)){
+	  if(debug){
+	    fprintf(stderr, "Ignoring plugin dir entry \"%s\""
+		    " with bad suffix %s\n", dirst->d_name, *suf);
+	  }
+	  bad_name = true;
+	  break;
+	}
+      }
+      
+      if(bad_name){
+	continue;
+      }
+    }
+    
+    char *filename = malloc(d_name_len + strlen(plugindir) + 2);
+    if (filename == NULL){
+      perror("malloc");
+      exitstatus = EXIT_FAILURE;
+      goto end;
+    }
+    strcpy(filename, plugindir); /* Spurious warning */
+    strcat(filename, "/");	/* Spurious warning */
+    strcat(filename, dirst->d_name); /* Spurious warning */
+    
+    ret = stat(filename, &st);
+    if (ret == -1){
+      perror("stat");
+      exitstatus = EXIT_FAILURE;
+      goto end;
+    }
+    
+    if (not S_ISREG(st.st_mode)	or (access(filename, X_OK) != 0)){
+      if(debug){
+	fprintf(stderr, "Ignoring plugin dir entry \"%s\""
+		" with bad type or mode\n", filename);
+      }
+      free(filename);
+      continue;
+    }
+    if(getplugin(dirst->d_name, &plugin_list)->disabled){
+      if(debug){
+	fprintf(stderr, "Ignoring disabled plugin \"%s\"\n",
+		dirst->d_name);
+      }
+      free(filename);
+      continue;
+    }
+    plugin *p = getplugin(dirst->d_name, &plugin_list);
+    {
+      /* Add global arguments to argument list for this plugin */
+      plugin *g = getplugin(NULL, &plugin_list);
+      for(char **a = g->argv + 1; *a != NULL; a++){
+	addargument(p, *a);
+      }
+    }
+    int pipefd[2]; 
+    ret = pipe(pipefd);
+    if (ret == -1){
+      perror("pipe");
+      exitstatus = EXIT_FAILURE;
+      goto end;
+    }
+    ret = set_cloexec_flag(pipefd[0]);
+    if(ret < 0){
+      perror("set_cloexec_flag");
+      exitstatus = EXIT_FAILURE;
+      goto end;
+    }
+    ret = set_cloexec_flag(pipefd[1]);
+    if(ret < 0){
+      perror("set_cloexec_flag");
+      exitstatus = EXIT_FAILURE;
+      goto end;
+    }
+    /* Block SIGCHLD until process is safely in process list */
+    ret = sigprocmask (SIG_BLOCK, &sigchld_action.sa_mask, NULL);
+    if(ret < 0){
+      perror("sigprocmask");
+      exitstatus = EXIT_FAILURE;
+      goto end;
+    }
+    // Starting a new process to be watched
+    pid_t pid = fork();
+    if(pid == 0){
+      /* this is the child process */
+      ret = sigaction(SIGCHLD, &old_sigchld_action, NULL);
+      if(ret < 0){
+	perror("sigaction");
+	_exit(EXIT_FAILURE);
+      }
+      ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
+      if(ret < 0){
+	perror("sigprocmask");
+	_exit(EXIT_FAILURE);
+      }
+
+      ret = dup2(pipefd[1], STDOUT_FILENO); /* replace our stdout */
+      if(ret == -1){
+	perror("dup2");
+	_exit(EXIT_FAILURE);
+      }
+      
+      if(dirfd(dir) < 0){
+	/* If dir has no file descriptor, we could not set FD_CLOEXEC
+	   above and must now close it manually here. */
+	closedir(dir);
+      }
+      if(execv(filename, p->argv) < 0){
+	perror("execv");
+	_exit(EXIT_FAILURE);
+      }
+      /* no return */
+    }
+    /* parent process */
+    free(filename);
+    close(pipefd[1]);		/* close unused write end of pipe */
+    process *new_process = malloc(sizeof(process));
+    if (new_process == NULL){
+      perror("malloc");
+      ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
+      if(ret < 0){
+	perror("sigprocmask");
+      }
+      exitstatus = EXIT_FAILURE;
+      goto end;
+    }
+    
+    *new_process = (struct process){ .pid = pid,
+				     .fd = pipefd[0],
+				     .next = process_list };
+    // List handling
+    process_list = new_process;
+    /* Unblock SIGCHLD so signal handler can be run if this process
+       has already completed */
+    ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
+    if(ret < 0){
+      perror("sigprocmask");
+      exitstatus = EXIT_FAILURE;
+      goto end;
+    }
+    
+    FD_SET(new_process->fd, &rfds_all);
+    
+    if (maxfd < new_process->fd){
+      maxfd = new_process->fd;
+    }
+    
+  }
+  
+  /* Free the plugin list */
+  for(plugin *next; plugin_list != NULL; plugin_list = next){
+    next = plugin_list->next;
+    free(plugin_list->argv);
+    free(plugin_list);
+  }
+  
+  closedir(dir);
+  dir = NULL;
+    
+  if (process_list == NULL){
+    fprintf(stderr, "No plugin processes started, exiting\n");
+    exitstatus = EXIT_FAILURE;
+    goto end;
+  }
+  while(process_list){
+    fd_set rfds = rfds_all;
+    int select_ret = select(maxfd+1, &rfds, NULL, NULL, NULL);
+    if (select_ret == -1){
+      perror("select");
+      exitstatus = EXIT_FAILURE;
+      goto end;
+    }
+    /* OK, now either a process completed, or something can be read
+       from one of them */
+    for(process *proc = process_list; proc ; proc = proc->next){
+      /* Is this process completely done? */
+      if(proc->eof and proc->completed){
+	/* Only accept the plugin output if it exited cleanly */
+	if(not WIFEXITED(proc->status)
+	   or WEXITSTATUS(proc->status) != 0){
+	  /* Bad exit by plugin */
+	  if(debug){
+	    if(WIFEXITED(proc->status)){
+	      fprintf(stderr, "Plugin %d exited with status %d\n",
+		      proc->pid, WEXITSTATUS(proc->status));
+	    } else if(WIFSIGNALED(proc->status)) {
+	      fprintf(stderr, "Plugin %d killed by signal %d\n",
+		      proc->pid, WTERMSIG(proc->status));
+	    } else if(WCOREDUMP(proc->status)){
+	      fprintf(stderr, "Plugin %d dumped core\n", proc->pid);
+	    }
+	  }
+	  /* Remove the plugin */
+	  FD_CLR(proc->fd, &rfds_all);
+	  /* Block signal while modifying process_list */
+	  ret = sigprocmask (SIG_BLOCK, &sigchld_action.sa_mask, NULL);
+	  if(ret < 0){
+	    perror("sigprocmask");
+	    exitstatus = EXIT_FAILURE;
+	    goto end;
+	  }
+	  /* Delete this process entry from the list */
+	  if(process_list == proc){
+	    /* First one - simple */
+	    process_list = proc->next;
+	  } else {
+	    /* Second one or later */
+	    for(process *p = process_list; p != NULL; p = p->next){
+	      if(p->next == proc){
+		p->next = proc->next;
+		break;
+	      }
+	    }
+	  }
+	  /* We are done modifying process list, so unblock signal */
+	  ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask,
+			     NULL);
+	  if(ret < 0){
+	    perror("sigprocmask");
+	  }
+	  free(proc->buffer);
+	  free(proc);
+	  /* We deleted this process from the list, so we can't go
+	     proc->next.  Therefore, start over from the beginning of
+	     the process list */
+	  break;
+	}
+	/* This process exited nicely, so print its buffer */
+	for(size_t written = 0; written < proc->buffer_length;
+	    written += (size_t)ret){
+	  ret = TEMP_FAILURE_RETRY(write(STDOUT_FILENO,
+					 proc->buffer + written,
+					 proc->buffer_length
+					 - written));
+	  if(ret < 0){
+	    perror("write");
+	    exitstatus = EXIT_FAILURE;
+	    goto end;
+	  }
+	}
+	goto end;
+      }
+      /* This process has not completed.  Does it have any output? */
+      if(proc->eof or not FD_ISSET(proc->fd, &rfds)){
+	/* This process had nothing to say at this time */
+	continue;
+      }
+      /* Before reading, make the process' data buffer large enough */
+      if(proc->buffer_length + BUFFER_SIZE > proc->buffer_size){
+	proc->buffer = realloc(proc->buffer, proc->buffer_size
+			       + (size_t) BUFFER_SIZE);
+	if (proc->buffer == NULL){
+	  perror("malloc");
+	  exitstatus = EXIT_FAILURE;
+	  goto end;
+	}
+	proc->buffer_size += BUFFER_SIZE;
+      }
+      /* Read from the process */
+      ret = read(proc->fd, proc->buffer + proc->buffer_length,
+		 BUFFER_SIZE);
+      if(ret < 0){
+	/* Read error from this process; ignore the error */
+	continue;
+      }
+      if(ret == 0){
+	/* got EOF */
+	proc->eof = true;
+      } else {
+	proc->buffer_length += (size_t) ret;
+      }
+    }
+  }
+  if(process_list == NULL){
+    fprintf(stderr, "All plugin processes failed, exiting\n");
+    exitstatus = EXIT_FAILURE;
+  }
+  
+ end:
+  /* Restore old signal handler */
+  sigaction(SIGCHLD, &old_sigchld_action, NULL);
+  
+  free(plus_argv);
+  
+  /* Free the plugin list */
+  for(plugin *next; plugin_list != NULL; plugin_list = next){
+    next = plugin_list->next;
+    free(plugin_list->argv);
+    free(plugin_list);
+  }
+  
+  if(dir != NULL){
+    closedir(dir);
+  }
+  
+  /* Free the process list and kill the processes */
+  for(process *next; process_list != NULL; process_list = next){
+    next = process_list->next;
+    close(process_list->fd);
+    ret = kill(process_list->pid, SIGTERM);
+    if(ret == -1 and errno != ESRCH){
+      /* set-uid proccesses migth not get closed */
+      perror("kill");
+    }
+    free(process_list->buffer);
+    free(process_list);
+  }
+  
+  /* Wait for any remaining child processes to terminate */
+  do{
+    ret = wait(NULL);
+  } while(ret >= 0);
+  if(errno != ECHILD){
+    perror("wait");
+  }
+  
+  return exitstatus;
+}

=== added file 'mandos-client.xml'
--- mandos-client.xml	1970-01-01 00:00:00 +0000
+++ mandos-client.xml	2008-08-08 01:31:58 +0000
@@ -0,0 +1,194 @@
+<?xml version='1.0' encoding='UTF-8'?>
+<?xml-stylesheet type="text/xsl"
+	href="http://docbook.sourceforge.net/release/xsl/current/manpages/docbook.xsl"?>
+<!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
+	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
+<!ENTITY VERSION "1.0">
+<!ENTITY COMMANDNAME "mandos-client">
+]>
+
+<refentry>
+  <refentryinfo>
+    <title>&COMMANDNAME;</title>
+    <!-- NWalsh's docbook scripts use this to generate the footer: -->
+    <productname>&COMMANDNAME;</productname>
+    <productnumber>&VERSION;</productnumber>
+    <authorgroup>
+      <author>
+	<firstname>Björn</firstname>
+	<surname>Påhlsson</surname>
+	<address>
+	  <email>belorn@fukt.bsnet.se</email>
+	</address>
+      </author>
+      <author>
+	<firstname>Teddy</firstname>
+	<surname>Hogeborn</surname>
+	<address>
+	  <email>teddy@fukt.bsnet.se</email>
+	</address>
+      </author>
+    </authorgroup>
+    <copyright>
+      <year>2008</year>
+      <holder>Teddy Hogeborn &amp; Björn Påhlsson</holder>
+    </copyright>
+    <legalnotice>
+      <para>
+	This manual page is free software: you can redistribute it
+	and/or modify it under the terms of the GNU General Public
+	License as published by the Free Software Foundation,
+	either version 3 of the License, or (at your option) any
+	later version.
+      </para>
+
+      <para>
+	This manual page is distributed in the hope that it will
+	be useful, but WITHOUT ANY WARRANTY; without even the
+	implied warranty of MERCHANTABILITY or FITNESS FOR A
+	PARTICULAR PURPOSE.  See the GNU General Public License
+	for more details.
+      </para>
+
+      <para>
+	You should have received a copy of the GNU General Public
+	License along with this program; If not, see
+	<ulink url="http://www.gnu.org/licenses/"/>.
+      </para>
+    </legalnotice>
+  </refentryinfo>
+
+  <refmeta>
+    <refentrytitle>&COMMANDNAME;</refentrytitle>
+    <manvolnum>8mandos</manvolnum>
+  </refmeta>
+  
+  <refnamediv>
+    <refname><command>&COMMANDNAME;</command></refname>
+    <refpurpose>
+      get password for encrypted rootdisk
+    </refpurpose>
+  </refnamediv>
+
+  <refsynopsisdiv>
+    <cmdsynopsis>
+      <command>&COMMANDNAME;</command>
+      <arg choice='opt' rep='repeat'>OPTION</arg>
+    </cmdsynopsis>
+  </refsynopsisdiv>
+
+  <refsect1 id="description">
+    <title>DESCRIPTION</title>
+    <para>
+      <command>&COMMANDNAME;</command> is a plugin runner that waits
+      for any of its plugins to return sucessfull with a password, and
+      passes it to cryptsetup. plugins is defined as any executables
+      in the plugins directory will by default will be
+      /conf/conf.d/mandos/plugins.d.  to stdout.
+    </para>
+
+    <variablelist>
+      <varlistentry>
+	<term><literal>-g</literal>,<literal>--global-options
+	<replaceable>OPTIONS</replaceable></literal></term>
+	<listitem>
+	  <para>
+	    Global options given to all plugins as additional start
+	    arguments.  Options are specified with a -o flag followed
+	    by a comma separated string of options.
+	  </para>	  
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>-o</literal>,<literal> --options-for
+	<replaceable>PLUGIN</replaceable>:<replaceable>OPTION</replaceable>
+	</literal></term>
+	<listitem>
+	  <para>
+	    Plugin specific options given to the plugin as additional
+	    start arguments.  Options are specified with a -o flag
+	    followed by a comma separated string of options.
+	  </para>	  
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>-d</literal>,<literal> --disable
+	<replaceable>PLUGIN</replaceable>
+	</literal></term>
+	<listitem>
+	  <para>
+	    Disable a specific plugin
+	  </para>	  
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>--groupid <replaceable>ID</replaceable>
+	</literal></term>
+	<listitem>
+	  <para>
+	    Group ID the plugins will run as
+	  </para>
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>--userid <replaceable>ID</replaceable>
+	</literal></term>
+	<listitem>
+	  <para>
+	    User ID the plugins will run as
+	  </para>
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>--plugin-dir <replaceable>DIRECTORY</replaceable>
+	</literal></term>
+	<listitem>
+	  <para>
+	    Specify a different plugin directory
+	  </para>
+	</listitem>
+      </varlistentry>       
+      
+      <varlistentry>
+	<term><literal>--debug</literal></term>
+	<listitem>
+	  <para>
+	    Debug mode
+	  </para>
+	</listitem>
+      </varlistentry>
+      
+      <varlistentry>
+	<term><literal>-?</literal>, <literal>--help</literal></term>
+	<listitem>
+	  <para>
+	    Gives a help message
+	  </para>
+	</listitem>
+      </varlistentry>
+      
+      <varlistentry>
+	<term><literal>--usage</literal></term>
+	<listitem>
+	  <para>
+	    Gives a short usage message
+	  </para>
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>-V</literal>, <literal>--version</literal></term>
+	<listitem>
+	  <para>
+	    Prints the program version
+	  </para>
+	</listitem>
+      </varlistentry>            
+    </variablelist>
+  </refsect1>
+</refentry>

=== added file 'mandos-clients.conf.xml'
--- mandos-clients.conf.xml	1970-01-01 00:00:00 +0000
+++ mandos-clients.conf.xml	2008-08-08 02:33:41 +0000
@@ -0,0 +1,250 @@
+<?xml version='1.0' encoding='UTF-8'?>
+<?xml-stylesheet type="text/xsl"
+	href="http://docbook.sourceforge.net/release/xsl/current/manpages/docbook.xsl"?>
+<!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
+	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
+<!ENTITY VERSION "1.0">
+<!ENTITY CONFNAME "mandos-clients.conf">
+<!ENTITY CONFPATH "<filename>/etc/mandos/clients.conf</filename>">
+]>
+
+<refentry>
+  <refentryinfo>
+    <title>&CONFNAME;</title>
+    <!-- NWalsh's docbook scripts use this to generate the footer: -->
+    <productname>&CONFNAME;</productname>
+    <productnumber>&VERSION;</productnumber>
+    <authorgroup>
+      <author>
+	<firstname>Björn</firstname>
+	<surname>Påhlsson</surname>
+	<address>
+	  <email>belorn@fukt.bsnet.se</email>
+	</address>
+      </author>
+      <author>
+	<firstname>Teddy</firstname>
+	<surname>Hogeborn</surname>
+	<address>
+	  <email>teddy@fukt.bsnet.se</email>
+	</address>
+      </author>
+    </authorgroup>
+    <copyright>
+      <year>2008</year>
+      <holder>Teddy Hogeborn &amp; Björn Påhlsson</holder>
+    </copyright>
+    <legalnotice>
+      <para>
+	This manual page is free software: you can redistribute it
+	and/or modify it under the terms of the GNU General Public
+	License as published by the Free Software Foundation,
+	either version 3 of the License, or (at your option) any
+	later version.
+      </para>
+
+      <para>
+	This manual page is distributed in the hope that it will
+	be useful, but WITHOUT ANY WARRANTY; without even the
+	implied warranty of MERCHANTABILITY or FITNESS FOR A
+	PARTICULAR PURPOSE.  See the GNU General Public License
+	for more details.
+      </para>
+
+      <para>
+	You should have received a copy of the GNU General Public
+	License along with this program; If not, see
+	<ulink url="http://www.gnu.org/licenses/"/>.
+      </para>
+    </legalnotice>
+  </refentryinfo>
+
+  <refmeta>
+    <refentrytitle>&CONFNAME;</refentrytitle>
+    <manvolnum>5</manvolnum>
+  </refmeta>
+  
+  <refnamediv>
+    <refname><filename>&CONFNAME;</filename></refname>
+    <refpurpose>
+      Configuration file for Mandos clients
+    </refpurpose>
+  </refnamediv>
+
+  <refsynopsisdiv>
+    <synopsis>
+      &CONFPATH;
+    </synopsis>
+  </refsynopsisdiv>
+
+  <refsect1 id="description">
+    <title>DESCRIPTION</title>
+    <para>
+      The file &CONFPATH; is the configuration file for mandos where
+      each client that will be abel to use the service need to be
+      specified. The configuration file is looked on at the startup of
+      the service, so to reenable timedout clients one need to only
+      restart the server. The format starts with a section under []
+      which is eather <literal>[DEFAULT]</literal> or a client
+      name. Values is set through the use of VAR = VALUE pair. Values
+      may not be empty.
+    </para>
+  </refsect1>
+
+  <refsect1 id="default">
+    <title>DEFAULTS</title>
+    <para>
+      The paramters for <literal>[DEFAULT]</literal> are:
+    </para>
+
+    <variablelist>
+      
+      <varlistentry>
+	<term><literal>timeout</literal></term>
+	<listitem>
+	  <para>
+	    This option allows you to override the default timeout
+	    that clients will get. By default mandos will use 1hr.
+	  </para>
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>interval</literal></term>
+	<listitem>
+	  <para>
+	    This option allows you to override the default interval
+	    used between checkups for disconnected clients. By default
+	    mandos will use 5m.
+	  </para>
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>checker</literal></term>
+	<listitem>
+	  <para>
+	    This option allows you to override the default shell
+	    command that the server will use to check up if the client
+	    is still up. By default mandos will "fping -q -- %%(fqdn)s"
+	  </para>
+	</listitem>
+      </varlistentry>
+      
+    </variablelist>
+  </refsect1>
+
+  <refsect1 id="clients">
+    <title>CLIENTS</title>
+    <para>
+      The paramters for clients are:
+    </para>
+
+    <variablelist>
+      
+      <varlistentry>
+	<term><literal>fingerprint</literal></term>
+	<listitem>
+	  <para>
+	    This option sets the openpgp fingerprint that identifies
+	    the public certificate that clients authenticates themself
+	    through gnutls. The string need to be in hex-decimal form.
+	  </para>
+	</listitem>
+      </varlistentry>
+      
+      <varlistentry>
+	<term><literal>secret</literal></term>
+	<listitem>
+	  <para>
+	    Base 64 encoded OpenPGP encrypted password encrypted by
+	    the clients openpgp certificate.
+	  </para>
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>secfile</literal></term>
+	<listitem>
+	  <para>
+	    Base 64 encoded OpenPGP encrypted password encrypted by
+	    the clients openpgp certificate as a binary file.
+	  </para>
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>fqdn</literal></term>
+	<listitem>
+	  <para>
+	    FQDN, that can be used in for checking that the client is up.
+	  </para>
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>checker</literal></term>
+	<listitem>
+	  <para>
+	    Shell command that the server will use to check up if a
+	    client is still up.
+	  </para>
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>timeout</literal></term>
+	<listitem>
+	  <para>
+	    Duration that a client can be down whitout be removed from
+	    the client list.
+	  </para>
+	</listitem>
+      </varlistentry> 
+      
+    </variablelist>
+  </refsect1>  
+
+  <refsect1 id="examples">
+    <title>EXAMPLES</title>
+    <informalexample>
+      <programlisting>
+[DEFAULT]
+timeout = 1h
+interval = 5m
+checker = fping -q -- %%(fqdn)s
+
+[example_client]
+fingerprint =  7788 2722 5BA7 DE53 9C5A  7CFA 59CF F7CD BD9A 5920
+
+secret =
+        hQIOA6QdEjBs2L/HEAf/TCyrDe5Xnm9esa+Pb/vWF9CUqfn4srzVgSu234
+        REJMVv7lBSrPE2132Lmd2gqF1HeLKDJRSVxJpt6xoWOChGHg+TMyXDxK+N
+        Xl89vGvdU1XfhKkVm9MDLOgT5ECDPysDGHFPDhqHOSu3Kaw2DWMV/iH9vz
+        3Z20erVNbdcvyBnuojcoWO/6yfB5EQO0BXp7kcyy00USA3CjD5FGZdoQGI
+        Tb8A/ar0tVA5crSQmaSotm6KmNLhrFnZ5BxX+TiE+eTUTqSloWRY6VAvqW
+        QHC7OASxK5E6RXPBuFH5IohUA2Qbk5AHt99pYvsIPX88j2rWauOokoiKZo
+        t/9leJ8VxO5l3wf/U64IH8bkPIoWmWZfd/nqh4uwGNbCgKMyT+AnvH7kMJ
+        3i7DivfWl2mKLV0PyPHUNva0VQxX6yYjcOhj1R6fCr/at8/NSLe2OhLchz
+        dC+Ls9h+kvJXgF8Sisv+Wk/1RadPLFmraRlqvJwt6Ww21LpiXqXHV2mIgq
+        WnR98YgSvUi3TJHrUQiNc9YyBzuRo0AjgG2C9qiE3FM+Y28+iQ/sR3+bFs
+        zYuZKVTObqiIslwXu7imO0cvvFRgJF/6u3HNFQ4LUTGhiM3FQmC6NNlF3/
+        vJM2hwRDMcJqDd54Twx90Wh+tYz0z7QMsK4ANXWHHWHR0JchnLWmenzbtW
+        5MHdW9AYsNJZAQSOpirE4Xi31CSlWAi9KV+cUCmWF5zOFy1x23P6PjdaRm
+        4T2zw4dxS5NswXWU0sVEXxjs6PYxuIiCTL7vdpx8QjBkrPWDrAbcMyBr2O
+        QlnHIvPzEArRQLo=
+        =iHhv
+
+fqdn = localhost
+interval = 5m
+      </programlisting>
+    </informalexample>
+  </refsect1>  
+  
+  <refsect1 id="files">
+    <title>FILES</title>
+    <para>
+      The file described here is &CONFPATH;
+    </para>
+  </refsect1>
+</refentry>

=== added file 'mandos.conf'
--- mandos.conf	1970-01-01 00:00:00 +0000
+++ mandos.conf	2008-08-08 01:17:17 +0000
@@ -0,0 +1,7 @@
+[server]
+#interface = 
+#address =
+#port = 
+#debug = False
+#priority = SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP
+#servicename = Mandos

=== added file 'mandos.conf.xml'
--- mandos.conf.xml	1970-01-01 00:00:00 +0000
+++ mandos.conf.xml	2008-08-08 01:31:58 +0000
@@ -0,0 +1,189 @@
+<?xml version='1.0' encoding='UTF-8'?>
+<?xml-stylesheet type="text/xsl"
+	href="http://docbook.sourceforge.net/release/xsl/current/manpages/docbook.xsl"?>
+<!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
+	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
+<!ENTITY VERSION "1.0">
+<!ENTITY CONFNAME "mandos.conf">
+<!ENTITY CONFPATH "<filename>/etc/mandos/mandos.conf</filename>">
+]>
+
+<refentry>
+  <refentryinfo>
+    <title>&CONFNAME;</title>
+    <!-- NWalsh's docbook scripts use this to generate the footer: -->
+    <productname>&CONFNAME;</productname>
+    <productnumber>&VERSION;</productnumber>
+    <authorgroup>
+      <author>
+	<firstname>Björn</firstname>
+	<surname>Påhlsson</surname>
+	<address>
+	  <email>belorn@fukt.bsnet.se</email>
+	</address>
+      </author>
+      <author>
+	<firstname>Teddy</firstname>
+	<surname>Hogeborn</surname>
+	<address>
+	  <email>teddy@fukt.bsnet.se</email>
+	</address>
+      </author>
+    </authorgroup>
+    <copyright>
+      <year>2008</year>
+      <holder>Teddy Hogeborn &amp; Björn Påhlsson</holder>
+    </copyright>
+    <legalnotice>
+      <para>
+	This manual page is free software: you can redistribute it
+	and/or modify it under the terms of the GNU General Public
+	License as published by the Free Software Foundation,
+	either version 3 of the License, or (at your option) any
+	later version.
+      </para>
+
+      <para>
+	This manual page is distributed in the hope that it will
+	be useful, but WITHOUT ANY WARRANTY; without even the
+	implied warranty of MERCHANTABILITY or FITNESS FOR A
+	PARTICULAR PURPOSE.  See the GNU General Public License
+	for more details.
+      </para>
+
+      <para>
+	You should have received a copy of the GNU General Public
+	License along with this program; If not, see
+	<ulink url="http://www.gnu.org/licenses/"/>.
+      </para>
+    </legalnotice>
+  </refentryinfo>
+
+  <refmeta>
+    <refentrytitle>&CONFNAME;</refentrytitle>
+    <manvolnum>5</manvolnum>
+  </refmeta>
+  
+  <refnamediv>
+    <refname><filename>&CONFNAME;</filename></refname>
+    <refpurpose>
+      Configuration file for Mandos
+    </refpurpose>
+  </refnamediv>
+
+  <refsynopsisdiv>
+    <synopsis>
+      &CONFPATH;
+    </synopsis>
+  </refsynopsisdiv>
+
+  <refsect1 id="description">
+    <title>DESCRIPTION</title>
+    <para>
+      The file &CONFPATH; is a simple configuration file for mandos
+      and is looked on at startup of the service. The configuration
+      file must start with <literal>[server]</literal>. The format for
+      the rest is a simple VAR = VALUE pair. Values may not be empty.
+    </para>
+
+    <para>
+      The paramters are:
+    </para>
+
+    <variablelist>
+      <varlistentry>
+	<term><literal>interface</literal></term>
+	<listitem>
+	  <para>
+	    This option allows you to override the default network
+	    interfaces. By default mandos will not bind to any
+	    specific interface but instead use default avahi-server
+	    behaviour.
+	  </para>
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>address</literal></term>
+	<listitem>
+	  <para>
+	    This option allows you to override the default network
+	    address. By default mandos will not bind to any
+	    specific address but instead use default avahi-server
+	    behaviour.
+	  </para>
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>port</literal></term>
+	<listitem>
+	  <para>
+	    This option allows you to override the default port to
+	    listen on. By default mandos will not specify any specific
+	    port and instead use a random port given by the OS from
+	    the use of INADDR_ANY.
+	  </para>
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>debug</literal></term>
+	<listitem>
+	  <para>
+	    This option allows you to modify debug mode with a true/false
+	    boolean value. By default is debug set to <literal>false</literal>.
+	  </para>
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>priority</literal></term>
+	<listitem>
+	  <para>
+	    This option allows you to override the default gnutls
+	    priority that will be used in gnutls session. See
+	    <citerefentry><refentrytitle>gnutls_priority_init
+	    </refentrytitle><manvolnum>3</manvolnum></citerefentry>for
+	    more information on gnutls priority strings.
+	  </para>	  
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>servicename</literal></term>
+	<listitem>
+	  <para>
+	    This option allows you to override the default Zeroconf
+	    service name use to announce mandos as a avahi service. By
+	    default mandos will use "Mandos".
+	  </para>
+	</listitem>
+      </varlistentry>
+      
+    </variablelist>
+  </refsect1>
+
+  <refsect1 id="examples">
+    <title>EXAMPLES</title>
+    <informalexample>
+      <programlisting>
+	[server]
+	# A configuration example
+	interface = eth0 
+	address = 2001:DB8:
+	port = 1025 
+	debug = true 
+	priority = SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP
+	servicename = Mandos
+      </programlisting>
+    </informalexample>
+  </refsect1>  
+  
+  <refsect1 id="files">
+    <title>FILES</title>
+    <para>
+      The file described here is &CONFPATH;
+    </para>
+  </refsect1>
+</refentry>

=== added file 'mandos.xml'
--- mandos.xml	1970-01-01 00:00:00 +0000
+++ mandos.xml	2008-08-08 01:51:58 +0000
@@ -0,0 +1,182 @@
+<?xml version='1.0' encoding='UTF-8'?>
+<?xml-stylesheet type="text/xsl"
+	href="http://docbook.sourceforge.net/release/xsl/current/manpages/docbook.xsl"?>
+<!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
+	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
+<!ENTITY VERSION "1.0">
+<!ENTITY COMMANDNAME "mandos">
+]>
+
+<refentry>
+  <refentryinfo>
+    <title>&COMMANDNAME;</title>
+    <!-- NWalsh's docbook scripts use this to generate the footer: -->
+    <productname>&COMMANDNAME;</productname>
+    <productnumber>&VERSION;</productnumber>
+    <authorgroup>
+      <author>
+	<firstname>Björn</firstname>
+	<surname>Påhlsson</surname>
+	<address>
+	  <email>belorn@fukt.bsnet.se</email>
+	</address>
+      </author>
+      <author>
+	<firstname>Teddy</firstname>
+	<surname>Hogeborn</surname>
+	<address>
+	  <email>teddy@fukt.bsnet.se</email>
+	</address>
+      </author>
+    </authorgroup>
+    <copyright>
+      <year>2008</year>
+      <holder>Teddy Hogeborn &amp; Björn Påhlsson</holder>
+    </copyright>
+    <legalnotice>
+      <para>
+	This manual page is free software: you can redistribute it
+	and/or modify it under the terms of the GNU General Public
+	License as published by the Free Software Foundation,
+	either version 3 of the License, or (at your option) any
+	later version.
+      </para>
+
+      <para>
+	This manual page is distributed in the hope that it will
+	be useful, but WITHOUT ANY WARRANTY; without even the
+	implied warranty of MERCHANTABILITY or FITNESS FOR A
+	PARTICULAR PURPOSE.  See the GNU General Public License
+	for more details.
+      </para>
+
+      <para>
+	You should have received a copy of the GNU General Public
+	License along with this program; If not, see
+	<ulink url="http://www.gnu.org/licenses/"/>.
+      </para>
+    </legalnotice>
+  </refentryinfo>
+
+  <refmeta>
+    <refentrytitle>&COMMANDNAME;</refentrytitle>
+    <manvolnum>8</manvolnum>
+  </refmeta>
+  
+  <refnamediv>
+    <refname><command>&COMMANDNAME;</command></refname>
+    <refpurpose>
+      Sends encrypted passwords to authenticated mandos clients
+    </refpurpose>
+  </refnamediv>
+
+  <refsynopsisdiv>
+    <cmdsynopsis>
+      <command>&COMMANDNAME;</command>
+      <arg choice='opt' rep='repeat'>OPTION</arg>
+    </cmdsynopsis>
+  </refsynopsisdiv>
+
+  <refsect1 id="description">
+    <title>DESCRIPTION</title>
+    <para>
+      <command>&COMMANDNAME;</command> is a server daemon that handels
+      incomming passwords request for passwords. Mandos use avahi to
+      announce the service, and through gnutls authenticates
+      clients. Any authenticated client is then given its encrypted
+      password.
+    </para>
+
+    <variablelist>
+      <varlistentry>
+	<term><literal>-h</literal>, <literal>--help</literal></term>
+	<listitem>
+	  <para>
+	    show a help message and exit
+	  </para>
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>-i</literal>, <literal>--interface <replaceable>
+	IF</replaceable></literal></term>
+	<listitem>
+	  <para>
+	    Bind to interface IF
+	  </para>
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>-a</literal>, <literal>--address <replaceable>
+	ADDRESS</replaceable></literal></term>
+	<listitem>
+	  <para>
+	    Address to listen for requests on
+	  </para>
+	</listitem>
+      </varlistentry>          
+
+      <varlistentry>
+	<term><literal>-p</literal>, <literal>--port <replaceable>
+	PORT</replaceable></literal></term>
+	<listitem>
+	  <para>
+	    Port number to receive requests on
+	  </para>
+	</listitem>
+      </varlistentry>          
+
+      <varlistentry>
+	<term><literal>--check</literal></term>
+	<listitem>
+	  <para>
+	    Run self-test on the server
+	  </para>
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>--debug</literal></term>
+	<listitem>
+	  <para>
+	    Debug mode
+	  </para>
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>--priority <replaceable>
+	PRIORITY</replaceable></literal></term>
+	<listitem>
+	  <para>
+	    GnuTLS priority string. See <citerefentry>
+	    <refentrytitle>gnutls_priority_init</refentrytitle>
+	    <manvolnum>3</manvolnum></citerefentry>
+	  </para>
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>--servicename <replaceable>NAME</replaceable>
+	</literal></term>
+	<listitem>
+	  <para>
+	    Zeroconf service name
+	  </para>
+	</listitem>
+      </varlistentry>     
+
+      <varlistentry>
+	<term><literal>--configdir <replaceable>DIR</replaceable>
+	</literal></term>
+	<listitem>
+	  <para>
+	    Directory to search for configuration files
+	  </para>
+	</listitem>
+      </varlistentry>
+      
+    </variablelist>
+  </refsect1>
+</refentry>

=== added file 'network-protocol.txt'
--- network-protocol.txt	1970-01-01 00:00:00 +0000
+++ network-protocol.txt	2008-08-07 22:30:45 +0000
@@ -0,0 +1,19 @@
+The Mandos server announces itself as a Zeroconf service of type
+"_mandos._tcp". The Mandos client sends a line of text where the first
+whitespace-separated field is the protocol version, which currently is
+"1".  The client and server then start a TLS protocol handshake with a
+slight quirk: the Mandos server program acts as a TLS "client" while
+the connecting Mandos client acts as a TLS "server".  The Mandos
+client must supply an OpenPGP certificate, and the fingerprint of this
+certificate is used by the Mandos server to look up (in a list read
+from a file at start time) which binary blob to give the client.  No
+other authentication or authorization is done by the server.
+
+| Mandos Client                              |     | Mandos Server |
+|--------------------------------------------+-----+---------------|
+| Connect                                    |     |               |
+| "1\r\n"                                    | ->  |               |
+| TLS handshake                              | <-> | TLS handshake |
+| OpenPGP public key (part of TLS handshake) | ->  |               |
+|                                            | <-  | Binary blob   |
+|                                            |     | Close         |

=== added directory 'plugins.d'
=== added file 'plugins.d/password-prompt.c'
--- plugins.d/password-prompt.c	1970-01-01 00:00:00 +0000
+++ plugins.d/password-prompt.c	2008-08-09 04:42:39 +0000
@@ -0,0 +1,221 @@
+/*  -*- coding: utf-8 -*- */
+/*
+ * Passprompt - Read a password from the terminal and print it
+ *
+ * Copyright © 2007-2008 Teddy Hogeborn & Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ * 
+ * Contact the authors at <https://www.fukt.bsnet.se/~belorn/> and
+ * <https://www.fukt.bsnet.se/~teddy/>.
+ */
+
+#define _GNU_SOURCE		/* getline() */
+
+#include <termios.h> 		/* struct termios, tcsetattr(),
+				   TCSAFLUSH, tcgetattr(), ECHO */
+#include <unistd.h>		/* struct termios, tcsetattr(),
+				   STDIN_FILENO, TCSAFLUSH,
+				   tcgetattr(), ECHO */
+#include <signal.h>		/* sig_atomic_t, raise(), struct
+				   sigaction, sigemptyset(),
+				   sigaction(), sigaddset(), SIGINT,
+				   SIGQUIT, SIGHUP, SIGTERM */
+#include <stddef.h>		/* NULL, size_t, ssize_t */
+#include <sys/types.h>		/* ssize_t */
+#include <stdlib.h>		/* EXIT_SUCCESS, EXIT_FAILURE,
+				   getopt_long */
+#include <stdio.h>		/* fprintf(), stderr, getline(),
+				   stdin, feof(), perror(), fputc(),
+				   stdout, getopt_long */
+#include <errno.h>		/* errno, EINVAL */
+#include <iso646.h>		/* or, not */
+#include <stdbool.h>		/* bool, false, true */
+#include <string.h> 		/* strlen, rindex, strncmp, strcmp */
+#include <argp.h>		/* struct argp_option, struct
+				   argp_state, struct argp,
+				   argp_parse(), error_t,
+				   ARGP_KEY_ARG, ARGP_KEY_END,
+				   ARGP_ERR_UNKNOWN */
+
+volatile bool quit_now = false;
+bool debug = false;
+const char *argp_program_version = "passprompt 0.9";
+const char *argp_program_bug_address = "<mandos@fukt.bsnet.se>";
+
+static void termination_handler(__attribute__((unused))int signum){
+  quit_now = true;
+}
+
+int main(int argc, char **argv){
+  ssize_t ret;
+  size_t n;
+  struct termios t_new, t_old;
+  char *buffer = NULL;
+  char *prefix = NULL;
+  int status = EXIT_SUCCESS;
+  struct sigaction old_action,
+    new_action = { .sa_handler = termination_handler,
+		   .sa_flags = 0 };
+  {
+    struct argp_option options[] = {
+      { .name = "prefix", .key = 'p',
+	.arg = "PREFIX", .flags = 0,
+	.doc = "Prefix used before the passprompt", .group = 2 },
+      { .name = "debug", .key = 128,
+	.doc = "Debug mode", .group = 3 },
+      { .name = NULL }
+    };
+  
+    error_t parse_opt (int key, char *arg, struct argp_state *state) {
+      /* Get the INPUT argument from `argp_parse', which we know is a
+	 pointer to our plugin list pointer. */
+      switch (key) {
+      case 'p':
+	prefix = arg;
+	break;
+      case 128:
+	debug = true;
+	break;
+      case ARGP_KEY_ARG:
+	argp_usage (state);
+	break;
+      case ARGP_KEY_END:
+	break;
+      default:
+	return ARGP_ERR_UNKNOWN;
+      }
+      return 0;
+    }
+  
+    struct argp argp = { .options = options, .parser = parse_opt,
+			 .args_doc = "",
+			 .doc = "Mandos Passprompt -- Provides a passprompt" };
+    ret = argp_parse (&argp, argc, argv, 0, 0, NULL);
+    if (ret == ARGP_ERR_UNKNOWN){
+      perror("argp_parse");
+      return EXIT_FAILURE;
+    }
+  }
+    
+  if (debug){
+    fprintf(stderr, "Starting %s\n", argv[0]);
+  }
+  if (debug){
+    fprintf(stderr, "Storing current terminal attributes\n");
+  }
+  
+  if (tcgetattr(STDIN_FILENO, &t_old) != 0){
+    return EXIT_FAILURE;
+  }
+  
+  sigemptyset(&new_action.sa_mask);
+  sigaddset(&new_action.sa_mask, SIGINT);
+  sigaddset(&new_action.sa_mask, SIGHUP);
+  sigaddset(&new_action.sa_mask, SIGTERM);
+  ret = sigaction(SIGINT, NULL, &old_action);
+  if(ret == -1){
+    perror("sigaction");
+    return EXIT_FAILURE;
+  }
+  if (old_action.sa_handler != SIG_IGN){
+    ret = sigaction(SIGINT, &new_action, NULL);
+    if(ret == -1){
+      perror("sigaction");
+      return EXIT_FAILURE;
+    }
+  }
+  ret = sigaction(SIGHUP, NULL, &old_action);
+  if(ret == -1){
+    perror("sigaction");
+    return EXIT_FAILURE;
+  }
+  if (old_action.sa_handler != SIG_IGN){
+    ret = sigaction(SIGHUP, &new_action, NULL);
+    if(ret == -1){
+      perror("sigaction");
+      return EXIT_FAILURE;
+    }
+  }
+  ret = sigaction(SIGTERM, NULL, &old_action);
+  if(ret == -1){
+    perror("sigaction");
+    return EXIT_FAILURE;
+  }
+  if (old_action.sa_handler != SIG_IGN){
+    ret = sigaction(SIGTERM, &new_action, NULL);
+    if(ret == -1){
+      perror("sigaction");
+      return EXIT_FAILURE;
+    }
+  }
+  
+  
+  if (debug){
+    fprintf(stderr, "Removing echo flag from terminal attributes\n");
+  }
+  
+  t_new = t_old;
+  t_new.c_lflag &= ~ECHO;
+  if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_new) != 0){
+    perror("tcsetattr-echo");
+    return EXIT_FAILURE;
+  }
+
+  if (debug){
+    fprintf(stderr, "Waiting for input from stdin \n");
+  }
+  while(true){
+    if (quit_now){
+      status = EXIT_FAILURE;
+      break;
+    }
+
+    if(prefix){
+      fprintf(stderr, "%s Password: ", prefix);
+    } else {
+      fprintf(stderr, "Password: ");
+    }      
+    ret = getline(&buffer, &n, stdin);
+    if (ret > 0){
+      fprintf(stdout, "%s", buffer);
+      status = EXIT_SUCCESS;
+      break;
+    }
+    if (ret < 0){
+      if (errno != EINTR and not feof(stdin)){
+	perror("getline");
+	status = EXIT_FAILURE;
+	break;
+      }
+    }
+    /* if(ret == 0), then the only sensible thing to do is to retry to
+       read from stdin */
+    fputc('\n', stderr);
+  }
+  
+  if (debug){
+    fprintf(stderr, "Restoring terminal attributes\n");
+  }
+  if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_old) != 0){
+    perror("tcsetattr+echo");
+  }
+  
+  if (debug){
+    fprintf(stderr, "%s is exiting\n", argv[0]);
+  }
+  
+  return status;
+}

=== added file 'plugins.d/password-prompt.xml'
--- plugins.d/password-prompt.xml	1970-01-01 00:00:00 +0000
+++ plugins.d/password-prompt.xml	2008-08-08 01:31:58 +0000
@@ -0,0 +1,140 @@
+<?xml version='1.0' encoding='UTF-8'?>
+<?xml-stylesheet type="text/xsl"
+	href="http://docbook.sourceforge.net/release/xsl/current/manpages/docbook.xsl"?>
+<!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
+	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
+<!ENTITY VERSION "1.0">
+<!ENTITY COMMANDNAME "password-prompt">
+]>
+
+<refentry>
+  <refentryinfo>
+    <title>&COMMANDNAME;</title>
+    <!-- NWalsh's docbook scripts use this to generate the footer: -->
+    <productname>&COMMANDNAME;</productname>
+    <productnumber>&VERSION;</productnumber>
+    <authorgroup>
+      <author>
+	<firstname>Björn</firstname>
+	<surname>Påhlsson</surname>
+	<address>
+	  <email>belorn@fukt.bsnet.se</email>
+	</address>
+      </author>
+      <author>
+	<firstname>Teddy</firstname>
+	<surname>Hogeborn</surname>
+	<address>
+	  <email>teddy@fukt.bsnet.se</email>
+	</address>
+      </author>
+    </authorgroup>
+    <copyright>
+      <year>2008</year>
+      <holder>Teddy Hogeborn &amp; Björn Påhlsson</holder>
+    </copyright>
+    <legalnotice>
+      <para>
+	This manual page is free software: you can redistribute it
+	and/or modify it under the terms of the GNU General Public
+	License as published by the Free Software Foundation,
+	either version 3 of the License, or (at your option) any
+	later version.
+      </para>
+
+      <para>
+	This manual page is distributed in the hope that it will
+	be useful, but WITHOUT ANY WARRANTY; without even the
+	implied warranty of MERCHANTABILITY or FITNESS FOR A
+	PARTICULAR PURPOSE.  See the GNU General Public License
+	for more details.
+      </para>
+
+      <para>
+	You should have received a copy of the GNU General Public
+	License along with this program; If not, see
+	<ulink url="http://www.gnu.org/licenses/"/>.
+      </para>
+    </legalnotice>
+  </refentryinfo>
+
+  <refmeta>
+    <refentrytitle>&COMMANDNAME;</refentrytitle>
+    <manvolnum>8mandos</manvolnum>
+  </refmeta>
+  
+  <refnamediv>
+    <refname><command>&COMMANDNAME;</command></refname>
+    <refpurpose>
+      Passprompt for luks during boot sequence
+    </refpurpose>
+  </refnamediv>
+
+  <refsynopsisdiv>
+    <cmdsynopsis>
+      <command>&COMMANDNAME;</command>
+      <arg choice='opt' rep='repeat'>OPTION</arg>
+    </cmdsynopsis>
+  </refsynopsisdiv>
+
+  <refsect1 id="description">
+    <title>DESCRIPTION</title>
+    <para>
+      <command>&COMMANDNAME;</command> is a terminal program that ask for
+      passwords during boot sequence. It is a plugin to
+      <firstterm>mandos</firstterm>, and is used as a fallback and
+      alternative to retriving passwords from a mandos server. During
+      boot sequence the user is prompted for the disk password, and
+      when a password is given it then gets forwarded to
+      <acronym>LUKS</acronym>.
+    </para>
+
+    <variablelist>
+      <varlistentry>
+	<term><literal>-p</literal>, <literal>--prefix=<replaceable>PREFIX
+	</replaceable></literal></term>
+	<listitem>
+	  <para>
+	    Prefix used before the passprompt
+	  </para>
+	</listitem>
+      </varlistentry>
+      
+      <varlistentry>
+	<term><literal>--debug</literal></term>
+	<listitem>
+	  <para>
+	    Debug mode
+	  </para>
+	</listitem>
+      </varlistentry>
+      
+      <varlistentry>
+	<term><literal>-?</literal>, <literal>--help</literal></term>
+	<listitem>
+	  <para>
+	    Gives a help message
+	  </para>
+	</listitem>
+      </varlistentry>
+      
+      <varlistentry>
+	<term><literal>--usage</literal></term>
+	<listitem>
+	  <para>
+	    Gives a short usage message
+	  </para>
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>-V</literal>, <literal>--version</literal></term>
+	<listitem>
+	  <para>
+	    Prints the program version
+	  </para>
+	</listitem>
+      </varlistentry>            
+    </variablelist>
+  </refsect1>
+</refentry>

=== added file 'plugins.d/password-request.c'
--- plugins.d/password-request.c	1970-01-01 00:00:00 +0000
+++ plugins.d/password-request.c	2008-08-09 04:42:39 +0000
@@ -0,0 +1,1077 @@
+/*  -*- coding: utf-8 -*- */
+/*
+ * Mandos client - get and decrypt data from a Mandos server
+ *
+ * This program is partly derived from an example program for an Avahi
+ * service browser, downloaded from
+ * <http://avahi.org/browser/examples/core-browse-services.c>.  This
+ * includes the following functions: "resolve_callback",
+ * "browse_callback", and parts of "main".
+ * 
+ * Everything else is
+ * Copyright © 2007-2008 Teddy Hogeborn & Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ * 
+ * Contact the authors at <mandos@fukt.bsnet.se>.
+ */
+
+/* Needed by GPGME, specifically gpgme_data_seek() */
+#define _LARGEFILE_SOURCE
+#define _FILE_OFFSET_BITS 64
+
+#define _GNU_SOURCE		/* TEMP_FAILURE_RETRY() */
+
+#include <stdio.h>		/* fprintf(), stderr, fwrite(), stdout, ferror() */
+#include <stdint.h> 		/* uint16_t, uint32_t */
+#include <stddef.h>		/* NULL, size_t, ssize_t */
+#include <stdlib.h> 		/* free() */
+#include <stdbool.h>		/* bool, true */
+#include <string.h>		/* memset(), strcmp(), strlen, strerror() */
+#include <sys/ioctl.h>          /* ioctl, ifreq, SIOCGIFFLAGS, IFF_UP,
+				   SIOCSIFFLAGS */
+#include <sys/types.h>		/* socket(), inet_pton(), sockaddr,
+				   sockaddr_in6, PF_INET6, SOCK_STREAM, INET6_ADDRSTRLEN */
+#include <sys/socket.h>		/* socket(), struct sockaddr_in6,
+				   struct in6_addr, inet_pton(),
+				   connect() */
+#include <assert.h>
+#include <errno.h>		/* perror() */
+#include <time.h>
+#include <net/if.h>		/* ioctl, ifreq, SIOCGIFFLAGS, IFF_UP,
+				   SIOCSIFFLAGS, if_indextoname(),
+				   if_nametoindex(), IF_NAMESIZE */
+#include <unistd.h>		/* close(), SEEK_SET, off_t, write()*/
+#include <netinet/in.h>
+#include <arpa/inet.h>		/* inet_pton(), htons */
+#include <iso646.h>		/* not */
+#include <argp.h>		/* struct argp_option,
+				   struct argp_state, struct argp,
+				   argp_parse() */
+
+/* Avahi */
+#include <avahi-core/core.h> 	/* AvahiSimplePoll, AvahiServer,
+				   AvahiIfIndex */
+#include <avahi-core/lookup.h>
+#include <avahi-core/log.h> 	/* AvahiLogLevel */
+#include <avahi-common/simple-watch.h>
+#include <avahi-common/malloc.h>
+#include <avahi-common/error.h>
+
+/* GnuTLS */
+#include <gnutls/gnutls.h>	/* gnutls_certificate_credentials_t,
+				   gnutls_dh_params_t,
+				   gnutls_strerror(),
+				   gnutls_global_init(),
+				   gnutls_global_set_log_level(),
+				   gnutls_global_set_log_function(),
+				   gnutls_certificate_allocate_credentials(),
+				   gnutls_global_deinit(),
+				   gnutls_dh_params_init(),
+				   gnutls_dh_params_generate(),
+				   gnutls_certificate_set_dh_params(),
+				   gnutls_certificate_free_credentials(),
+				   gnutls_session_t, gnutls_init(),
+				   gnutls_priority_set_direct(),
+				   gnutls_deinit(),
+				   gnutls_credentials_set(),
+				   gnutls_certificate_server_set_request(),
+				   gnutls_dh_set_prime_bits(),
+				   gnutls_transport_set_ptr(),
+				   gnutls_transport_ptr_t,
+				   gnutls_handshake(),
+				   gnutls_record_recv()
+				   gnutls_perror(), gnutls_bye(),
+				   init_gnutls_session(),
+				   GNUTLS_E_SUCCESS,
+				   GNUTLS_CRD_CERTIFICATE,
+				   GNUTLS_CERT_IGNORE,
+				   GNUTLS_E_INTERRUPTED,
+				   GNUTLS_E_AGAIN,
+				   GNUTLS_E_REHANDSHAKE,
+				   GNUTLS_SHUT_RDWR, */
+#include <gnutls/openpgp.h> /* gnutls_certificate_set_openpgp_key_file(),
+			       GNUTLS_OPENPGP_FMT_BASE64 */
+
+/* GPGME */
+#include <gpgme.h> 		/* gpgme_data_t, gpgme_ctx_t,
+				   gpgme_error_t, gpgme_engine_info_t,
+				   gpgme_check_version(),
+				   gpgme_engine_check_version(),
+				   gpgme_strsource(),
+				   gpgme_strerror(),
+				   gpgme_get_engine_info(),
+				   gpgme_set_engine_info(),
+				   gpgme_data_new_from_mem(),
+				   gpgme_data_new(), gpgme_new(),
+				   gpgme_op_decrypt(),
+				   gpgme_decrypt_result_t,
+				   gpgme_op_decrypt_result(),
+				   gpgme_recipient_t,
+				   gpgme_pubkey_algo_name(),
+				   gpgme_data_seek(),
+				   gpgme_data_read(),
+				   gpgme_data_release()
+				   GPGME_PROTOCOL_OpenPGP,
+				   GPG_ERR_NO_ERROR,
+				   GPG_ERR_NO_SECKEY, */
+
+#define BUFFER_SIZE 256
+
+bool debug = false;
+static const char *keydir = "/conf/conf.d/mandos";
+static const char mandos_protocol_version[] = "1";
+const char *argp_program_version = "mandosclient 0.9";
+const char *argp_program_bug_address = "<mandos@fukt.bsnet.se>";
+
+/* Used for passing in values through the Avahi callback functions */
+typedef struct {
+  AvahiSimplePoll *simple_poll;
+  AvahiServer *server;
+  gnutls_certificate_credentials_t cred;
+  unsigned int dh_bits;
+  gnutls_dh_params_t dh_params;
+  const char *priority;
+} mandos_context;
+
+/*
+ * Make room in "buffer" for at least BUFFER_SIZE additional bytes.
+ * "buffer_capacity" is how much is currently allocated,
+ * "buffer_length" is how much is already used.
+ */
+size_t adjustbuffer(char **buffer, size_t buffer_length,
+		  size_t buffer_capacity){
+  if (buffer_length + BUFFER_SIZE > buffer_capacity){
+    *buffer = realloc(*buffer, buffer_capacity + BUFFER_SIZE);
+    if (buffer == NULL){
+      return 0;
+    }
+    buffer_capacity += BUFFER_SIZE;
+  }
+  return buffer_capacity;
+}
+
+/* 
+ * Decrypt OpenPGP data using keyrings in HOMEDIR.
+ * Returns -1 on error
+ */
+static ssize_t pgp_packet_decrypt (const char *cryptotext,
+				   size_t crypto_size,
+				   char **plaintext,
+				   const char *homedir){
+  gpgme_data_t dh_crypto, dh_plain;
+  gpgme_ctx_t ctx;
+  gpgme_error_t rc;
+  ssize_t ret;
+  size_t plaintext_capacity = 0;
+  ssize_t plaintext_length = 0;
+  gpgme_engine_info_t engine_info;
+  
+  if (debug){
+    fprintf(stderr, "Trying to decrypt OpenPGP data\n");
+  }
+  
+  /* Init GPGME */
+  gpgme_check_version(NULL);
+  rc = gpgme_engine_check_version(GPGME_PROTOCOL_OpenPGP);
+  if (rc != GPG_ERR_NO_ERROR){
+    fprintf(stderr, "bad gpgme_engine_check_version: %s: %s\n",
+	    gpgme_strsource(rc), gpgme_strerror(rc));
+    return -1;
+  }
+  
+  /* Set GPGME home directory for the OpenPGP engine only */
+  rc = gpgme_get_engine_info (&engine_info);
+  if (rc != GPG_ERR_NO_ERROR){
+    fprintf(stderr, "bad gpgme_get_engine_info: %s: %s\n",
+	    gpgme_strsource(rc), gpgme_strerror(rc));
+    return -1;
+  }
+  while(engine_info != NULL){
+    if(engine_info->protocol == GPGME_PROTOCOL_OpenPGP){
+      gpgme_set_engine_info(GPGME_PROTOCOL_OpenPGP,
+			    engine_info->file_name, homedir);
+      break;
+    }
+    engine_info = engine_info->next;
+  }
+  if(engine_info == NULL){
+    fprintf(stderr, "Could not set GPGME home dir to %s\n", homedir);
+    return -1;
+  }
+  
+  /* Create new GPGME data buffer from memory cryptotext */
+  rc = gpgme_data_new_from_mem(&dh_crypto, cryptotext, crypto_size,
+			       0);
+  if (rc != GPG_ERR_NO_ERROR){
+    fprintf(stderr, "bad gpgme_data_new_from_mem: %s: %s\n",
+	    gpgme_strsource(rc), gpgme_strerror(rc));
+    return -1;
+  }
+  
+  /* Create new empty GPGME data buffer for the plaintext */
+  rc = gpgme_data_new(&dh_plain);
+  if (rc != GPG_ERR_NO_ERROR){
+    fprintf(stderr, "bad gpgme_data_new: %s: %s\n",
+	    gpgme_strsource(rc), gpgme_strerror(rc));
+    gpgme_data_release(dh_crypto);
+    return -1;
+  }
+  
+  /* Create new GPGME "context" */
+  rc = gpgme_new(&ctx);
+  if (rc != GPG_ERR_NO_ERROR){
+    fprintf(stderr, "bad gpgme_new: %s: %s\n",
+	    gpgme_strsource(rc), gpgme_strerror(rc));
+    plaintext_length = -1;
+    goto decrypt_end;
+  }
+  
+  /* Decrypt data from the cryptotext data buffer to the plaintext
+     data buffer */
+  rc = gpgme_op_decrypt(ctx, dh_crypto, dh_plain);
+  if (rc != GPG_ERR_NO_ERROR){
+    fprintf(stderr, "bad gpgme_op_decrypt: %s: %s\n",
+	    gpgme_strsource(rc), gpgme_strerror(rc));
+    plaintext_length = -1;
+    goto decrypt_end;
+  }
+  
+  if(debug){
+    fprintf(stderr, "Decryption of OpenPGP data succeeded\n");
+  }
+  
+  if (debug){
+    gpgme_decrypt_result_t result;
+    result = gpgme_op_decrypt_result(ctx);
+    if (result == NULL){
+      fprintf(stderr, "gpgme_op_decrypt_result failed\n");
+    } else {
+      fprintf(stderr, "Unsupported algorithm: %s\n",
+	      result->unsupported_algorithm);
+      fprintf(stderr, "Wrong key usage: %d\n",
+	      result->wrong_key_usage);
+      if(result->file_name != NULL){
+	fprintf(stderr, "File name: %s\n", result->file_name);
+      }
+      gpgme_recipient_t recipient;
+      recipient = result->recipients;
+      if(recipient){
+	while(recipient != NULL){
+	  fprintf(stderr, "Public key algorithm: %s\n",
+		  gpgme_pubkey_algo_name(recipient->pubkey_algo));
+	  fprintf(stderr, "Key ID: %s\n", recipient->keyid);
+	  fprintf(stderr, "Secret key available: %s\n",
+		  recipient->status == GPG_ERR_NO_SECKEY
+		  ? "No" : "Yes");
+	  recipient = recipient->next;
+	}
+      }
+    }
+  }
+  
+  /* Seek back to the beginning of the GPGME plaintext data buffer */
+  if (gpgme_data_seek(dh_plain, (off_t) 0, SEEK_SET) == -1){
+    perror("pgpme_data_seek");
+    plaintext_length = -1;
+    goto decrypt_end;
+  }
+  
+  *plaintext = NULL;
+  while(true){
+    plaintext_capacity = adjustbuffer(plaintext,
+				      (size_t)plaintext_length,
+				      plaintext_capacity);
+    if (plaintext_capacity == 0){
+	perror("adjustbuffer");
+	plaintext_length = -1;
+	goto decrypt_end;
+    }
+    
+    ret = gpgme_data_read(dh_plain, *plaintext + plaintext_length,
+			  BUFFER_SIZE);
+    /* Print the data, if any */
+    if (ret == 0){
+      /* EOF */
+      break;
+    }
+    if(ret < 0){
+      perror("gpgme_data_read");
+      plaintext_length = -1;
+      goto decrypt_end;
+    }
+    plaintext_length += ret;
+  }
+
+  if(debug){
+    fprintf(stderr, "Decrypted password is: ");
+    for(ssize_t i = 0; i < plaintext_length; i++){
+      fprintf(stderr, "%02hhX ", (*plaintext)[i]);
+    }
+    fprintf(stderr, "\n");
+  }
+  
+ decrypt_end:
+  
+  /* Delete the GPGME cryptotext data buffer */
+  gpgme_data_release(dh_crypto);
+  
+  /* Delete the GPGME plaintext data buffer */
+  gpgme_data_release(dh_plain);
+  return plaintext_length;
+}
+
+static const char * safer_gnutls_strerror (int value) {
+  const char *ret = gnutls_strerror (value);
+  if (ret == NULL)
+    ret = "(unknown)";
+  return ret;
+}
+
+/* GnuTLS log function callback */
+static void debuggnutls(__attribute__((unused)) int level,
+			const char* string){
+  fprintf(stderr, "GnuTLS: %s", string);
+}
+
+static int init_gnutls_global(mandos_context *mc,
+			      const char *pubkeyfile,
+			      const char *seckeyfile){
+  int ret;
+  
+  if(debug){
+    fprintf(stderr, "Initializing GnuTLS\n");
+  }
+
+  if ((ret = gnutls_global_init ())
+      != GNUTLS_E_SUCCESS) {
+    fprintf (stderr, "GnuTLS global_init: %s\n",
+	     safer_gnutls_strerror(ret));
+    return -1;
+  }
+  
+  if (debug){
+    /* "Use a log level over 10 to enable all debugging options."
+     * - GnuTLS manual
+     */
+    gnutls_global_set_log_level(11);
+    gnutls_global_set_log_function(debuggnutls);
+  }
+  
+  /* OpenPGP credentials */
+  if ((ret = gnutls_certificate_allocate_credentials (&mc->cred))
+      != GNUTLS_E_SUCCESS) {
+    fprintf (stderr, "GnuTLS memory error: %s\n",
+	     safer_gnutls_strerror(ret));
+    gnutls_global_deinit ();
+    return -1;
+  }
+  
+  if(debug){
+    fprintf(stderr, "Attempting to use OpenPGP certificate %s"
+	    " and keyfile %s as GnuTLS credentials\n", pubkeyfile,
+	    seckeyfile);
+  }
+  
+  ret = gnutls_certificate_set_openpgp_key_file
+    (mc->cred, pubkeyfile, seckeyfile, GNUTLS_OPENPGP_FMT_BASE64);
+  if (ret != GNUTLS_E_SUCCESS) {
+    fprintf(stderr,
+	    "Error[%d] while reading the OpenPGP key pair ('%s',"
+	    " '%s')\n", ret, pubkeyfile, seckeyfile);
+    fprintf(stdout, "The GnuTLS error is: %s\n",
+	    safer_gnutls_strerror(ret));
+    goto globalfail;
+  }
+  
+  /* GnuTLS server initialization */
+  ret = gnutls_dh_params_init(&mc->dh_params);
+  if (ret != GNUTLS_E_SUCCESS) {
+    fprintf (stderr, "Error in GnuTLS DH parameter initialization:"
+	     " %s\n", safer_gnutls_strerror(ret));
+    goto globalfail;
+  }
+  ret = gnutls_dh_params_generate2(mc->dh_params, mc->dh_bits);
+  if (ret != GNUTLS_E_SUCCESS) {
+    fprintf (stderr, "Error in GnuTLS prime generation: %s\n",
+	     safer_gnutls_strerror(ret));
+    goto globalfail;
+  }
+  
+  gnutls_certificate_set_dh_params(mc->cred, mc->dh_params);
+
+  return 0;
+
+ globalfail:
+
+  gnutls_certificate_free_credentials(mc->cred);
+  gnutls_global_deinit();
+  return -1;
+
+}
+
+static int init_gnutls_session(mandos_context *mc,
+			       gnutls_session_t *session){
+  int ret;
+  /* GnuTLS session creation */
+  ret = gnutls_init(session, GNUTLS_SERVER);
+  if (ret != GNUTLS_E_SUCCESS){
+    fprintf(stderr, "Error in GnuTLS session initialization: %s\n",
+	    safer_gnutls_strerror(ret));
+  }
+  
+  {
+    const char *err;
+    ret = gnutls_priority_set_direct(*session, mc->priority, &err);
+    if (ret != GNUTLS_E_SUCCESS) {
+      fprintf(stderr, "Syntax error at: %s\n", err);
+      fprintf(stderr, "GnuTLS error: %s\n",
+	      safer_gnutls_strerror(ret));
+      gnutls_deinit (*session);
+      return -1;
+    }
+  }
+  
+  ret = gnutls_credentials_set(*session, GNUTLS_CRD_CERTIFICATE,
+			       mc->cred);
+  if (ret != GNUTLS_E_SUCCESS) {
+    fprintf(stderr, "Error setting GnuTLS credentials: %s\n",
+	    safer_gnutls_strerror(ret));
+    gnutls_deinit (*session);
+    return -1;
+  }
+  
+  /* ignore client certificate if any. */
+  gnutls_certificate_server_set_request (*session,
+					 GNUTLS_CERT_IGNORE);
+  
+  gnutls_dh_set_prime_bits (*session, mc->dh_bits);
+  
+  return 0;
+}
+
+/* Avahi log function callback */
+static void empty_log(__attribute__((unused)) AvahiLogLevel level,
+		      __attribute__((unused)) const char *txt){}
+
+/* Called when a Mandos server is found */
+static int start_mandos_communication(const char *ip, uint16_t port,
+				      AvahiIfIndex if_index,
+				      mandos_context *mc){
+  int ret, tcp_sd;
+  union { struct sockaddr in; struct sockaddr_in6 in6; } to;
+  char *buffer = NULL;
+  char *decrypted_buffer;
+  size_t buffer_length = 0;
+  size_t buffer_capacity = 0;
+  ssize_t decrypted_buffer_size;
+  size_t written;
+  int retval = 0;
+  char interface[IF_NAMESIZE];
+  gnutls_session_t session;
+  
+  ret = init_gnutls_session (mc, &session);
+  if (ret != 0){
+    return -1;
+  }
+  
+  if(debug){
+    fprintf(stderr, "Setting up a tcp connection to %s, port %d\n",
+	    ip, port);
+  }
+  
+  tcp_sd = socket(PF_INET6, SOCK_STREAM, 0);
+  if(tcp_sd < 0) {
+    perror("socket");
+    return -1;
+  }
+
+  if(debug){
+    if(if_indextoname((unsigned int)if_index, interface) == NULL){
+      perror("if_indextoname");
+      return -1;
+    }
+    fprintf(stderr, "Binding to interface %s\n", interface);
+  }
+  
+  memset(&to,0,sizeof(to));	/* Spurious warning */
+  to.in6.sin6_family = AF_INET6;
+  /* It would be nice to have a way to detect if we were passed an
+     IPv4 address here.   Now we assume an IPv6 address. */
+  ret = inet_pton(AF_INET6, ip, &to.in6.sin6_addr);
+  if (ret < 0 ){
+    perror("inet_pton");
+    return -1;
+  }
+  if(ret == 0){
+    fprintf(stderr, "Bad address: %s\n", ip);
+    return -1;
+  }
+  to.in6.sin6_port = htons(port);	/* Spurious warning */
+  
+  to.in6.sin6_scope_id = (uint32_t)if_index;
+  
+  if(debug){
+    fprintf(stderr, "Connection to: %s, port %d\n", ip, port);
+    char addrstr[INET6_ADDRSTRLEN] = "";
+    if(inet_ntop(to.in6.sin6_family, &(to.in6.sin6_addr), addrstr,
+		 sizeof(addrstr)) == NULL){
+      perror("inet_ntop");
+    } else {
+      if(strcmp(addrstr, ip) != 0){
+	fprintf(stderr, "Canonical address form: %s\n", addrstr);
+      }
+    }
+  }
+  
+  ret = connect(tcp_sd, &to.in, sizeof(to));
+  if (ret < 0){
+    perror("connect");
+    return -1;
+  }
+
+  const char *out = mandos_protocol_version;
+  written = 0;
+  while (true){
+    size_t out_size = strlen(out);
+    ret = TEMP_FAILURE_RETRY(write(tcp_sd, out + written,
+				   out_size - written));
+    if (ret == -1){
+      perror("write");
+      retval = -1;
+      goto mandos_end;
+    }
+    written += (size_t)ret;
+    if(written < out_size){
+      continue;
+    } else {
+      if (out == mandos_protocol_version){
+	written = 0;
+	out = "\r\n";
+      } else {
+	break;
+      }
+    }
+  }
+ 
+  if(debug){
+    fprintf(stderr, "Establishing TLS session with %s\n", ip);
+  }
+  
+  gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t) tcp_sd);
+  
+  ret = gnutls_handshake (session);
+  
+  if (ret != GNUTLS_E_SUCCESS){
+    if(debug){
+      fprintf(stderr, "*** GnuTLS Handshake failed ***\n");
+      gnutls_perror (ret);
+    }
+    retval = -1;
+    goto mandos_end;
+  }
+  
+  /* Read OpenPGP packet that contains the wanted password */
+  
+  if(debug){
+    fprintf(stderr, "Retrieving pgp encrypted password from %s\n",
+	    ip);
+  }
+
+  while(true){
+    buffer_capacity = adjustbuffer(&buffer, buffer_length,
+				   buffer_capacity);
+    if (buffer_capacity == 0){
+      perror("adjustbuffer");
+      retval = -1;
+      goto mandos_end;
+    }
+    
+    ret = gnutls_record_recv(session, buffer+buffer_length,
+			     BUFFER_SIZE);
+    if (ret == 0){
+      break;
+    }
+    if (ret < 0){
+      switch(ret){
+      case GNUTLS_E_INTERRUPTED:
+      case GNUTLS_E_AGAIN:
+	break;
+      case GNUTLS_E_REHANDSHAKE:
+	ret = gnutls_handshake (session);
+	if (ret < 0){
+	  fprintf(stderr, "*** GnuTLS Re-handshake failed ***\n");
+	  gnutls_perror (ret);
+	  retval = -1;
+	  goto mandos_end;
+	}
+	break;
+      default:
+	fprintf(stderr, "Unknown error while reading data from"
+		" encrypted session with Mandos server\n");
+	retval = -1;
+	gnutls_bye (session, GNUTLS_SHUT_RDWR);
+	goto mandos_end;
+      }
+    } else {
+      buffer_length += (size_t) ret;
+    }
+  }
+  
+  if(debug){
+    fprintf(stderr, "Closing TLS session\n");
+  }
+  
+  gnutls_bye (session, GNUTLS_SHUT_RDWR);
+  
+  if (buffer_length > 0){
+    decrypted_buffer_size = pgp_packet_decrypt(buffer,
+					       buffer_length,
+					       &decrypted_buffer,
+					       keydir);
+    if (decrypted_buffer_size >= 0){
+      written = 0;
+      while(written < (size_t) decrypted_buffer_size){
+	ret = (int)fwrite (decrypted_buffer + written, 1,
+			   (size_t)decrypted_buffer_size - written,
+			   stdout);
+	if(ret == 0 and ferror(stdout)){
+	  if(debug){
+	    fprintf(stderr, "Error writing encrypted data: %s\n",
+		    strerror(errno));
+	  }
+	  retval = -1;
+	  break;
+	}
+	written += (size_t)ret;
+      }
+      free(decrypted_buffer);
+    } else {
+      retval = -1;
+    }
+  }
+  
+  /* Shutdown procedure */
+  
+ mandos_end:
+  free(buffer);
+  close(tcp_sd);
+  gnutls_deinit (session);
+  return retval;
+}
+
+static void resolve_callback(AvahiSServiceResolver *r,
+			     AvahiIfIndex interface,
+			     AVAHI_GCC_UNUSED AvahiProtocol protocol,
+			     AvahiResolverEvent event,
+			     const char *name,
+			     const char *type,
+			     const char *domain,
+			     const char *host_name,
+			     const AvahiAddress *address,
+			     uint16_t port,
+			     AVAHI_GCC_UNUSED AvahiStringList *txt,
+			     AVAHI_GCC_UNUSED AvahiLookupResultFlags
+			     flags,
+			     void* userdata) {
+  mandos_context *mc = userdata;
+  assert(r);			/* Spurious warning */
+  
+  /* Called whenever a service has been resolved successfully or
+     timed out */
+  
+  switch (event) {
+  default:
+  case AVAHI_RESOLVER_FAILURE:
+    fprintf(stderr, "(Avahi Resolver) Failed to resolve service '%s'"
+	    " of type '%s' in domain '%s': %s\n", name, type, domain,
+	    avahi_strerror(avahi_server_errno(mc->server)));
+    break;
+    
+  case AVAHI_RESOLVER_FOUND:
+    {
+      char ip[AVAHI_ADDRESS_STR_MAX];
+      avahi_address_snprint(ip, sizeof(ip), address);
+      if(debug){
+	fprintf(stderr, "Mandos server \"%s\" found on %s (%s, %d) on"
+		" port %d\n", name, host_name, ip, interface, port);
+      }
+      int ret = start_mandos_communication(ip, port, interface, mc);
+      if (ret == 0){
+	exit(EXIT_SUCCESS);
+      }
+    }
+  }
+  avahi_s_service_resolver_free(r);
+}
+
+static void browse_callback( AvahiSServiceBrowser *b,
+			     AvahiIfIndex interface,
+			     AvahiProtocol protocol,
+			     AvahiBrowserEvent event,
+			     const char *name,
+			     const char *type,
+			     const char *domain,
+			     AVAHI_GCC_UNUSED AvahiLookupResultFlags
+			     flags,
+			     void* userdata) {
+  mandos_context *mc = userdata;
+  assert(b);			/* Spurious warning */
+  
+  /* Called whenever a new services becomes available on the LAN or
+     is removed from the LAN */
+  
+  switch (event) {
+  default:
+  case AVAHI_BROWSER_FAILURE:
+    
+    fprintf(stderr, "(Avahi browser) %s\n",
+	    avahi_strerror(avahi_server_errno(mc->server)));
+    avahi_simple_poll_quit(mc->simple_poll);
+    return;
+    
+  case AVAHI_BROWSER_NEW:
+    /* We ignore the returned Avahi resolver object. In the callback
+       function we free it. If the Avahi server is terminated before
+       the callback function is called the Avahi server will free the
+       resolver for us. */
+    
+    if (!(avahi_s_service_resolver_new(mc->server, interface,
+				       protocol, name, type, domain,
+				       AVAHI_PROTO_INET6, 0,
+				       resolve_callback, mc)))
+      fprintf(stderr, "Avahi: Failed to resolve service '%s': %s\n",
+	      name, avahi_strerror(avahi_server_errno(mc->server)));
+    break;
+    
+  case AVAHI_BROWSER_REMOVE:
+    break;
+    
+  case AVAHI_BROWSER_ALL_FOR_NOW:
+  case AVAHI_BROWSER_CACHE_EXHAUSTED:
+    if(debug){
+      fprintf(stderr, "No Mandos server found, still searching...\n");
+    }
+    break;
+  }
+}
+
+/* Combines file name and path and returns the malloced new
+   string. some sane checks could/should be added */
+static const char *combinepath(const char *first, const char *second){
+  size_t f_len = strlen(first);
+  size_t s_len = strlen(second);
+  char *tmp = malloc(f_len + s_len + 2);
+  if (tmp == NULL){
+    return NULL;
+  }
+  if(f_len > 0){
+    memcpy(tmp, first, f_len);	/* Spurious warning */
+  }
+  tmp[f_len] = '/';
+  if(s_len > 0){
+    memcpy(tmp + f_len + 1, second, s_len); /* Spurious warning */
+  }
+  tmp[f_len + 1 + s_len] = '\0';
+  return tmp;
+}
+
+
+int main(int argc, char *argv[]){
+    AvahiSServiceBrowser *sb = NULL;
+    int error;
+    int ret;
+    int exitcode = EXIT_SUCCESS;
+    const char *interface = "eth0";
+    struct ifreq network;
+    int sd;
+    uid_t uid;
+    gid_t gid;
+    char *connect_to = NULL;
+    AvahiIfIndex if_index = AVAHI_IF_UNSPEC;
+    const char *pubkeyfile = "pubkey.txt";
+    const char *seckeyfile = "seckey.txt";
+    mandos_context mc = { .simple_poll = NULL, .server = NULL,
+			  .dh_bits = 1024, .priority = "SECURE256"};
+    bool gnutls_initalized = false;
+    
+    {
+      struct argp_option options[] = {
+	{ .name = "debug", .key = 128,
+	  .doc = "Debug mode", .group = 3 },
+	{ .name = "connect", .key = 'c',
+	  .arg = "IP",
+	  .doc = "Connect directly to a sepcified mandos server",
+	  .group = 1 },
+	{ .name = "interface", .key = 'i',
+	  .arg = "INTERFACE",
+	  .doc = "Interface that Avahi will conntect through",
+	  .group = 1 },
+	{ .name = "keydir", .key = 'd',
+	  .arg = "KEYDIR",
+	  .doc = "Directory where the openpgp keyring is",
+	  .group = 1 },
+	{ .name = "seckey", .key = 's',
+	  .arg = "SECKEY",
+	  .doc = "Secret openpgp key for gnutls authentication",
+	  .group = 1 },
+	{ .name = "pubkey", .key = 'p',
+	  .arg = "PUBKEY",
+	  .doc = "Public openpgp key for gnutls authentication",
+	  .group = 2 },
+	{ .name = "dh-bits", .key = 129,
+	  .arg = "BITS",
+	  .doc = "dh-bits to use in gnutls communication",
+	  .group = 2 },
+	{ .name = "priority", .key = 130,
+	  .arg = "PRIORITY",
+	  .doc = "GNUTLS priority", .group = 1 },
+	{ .name = NULL }
+      };
+
+      
+      error_t parse_opt (int key, char *arg,
+			 struct argp_state *state) {
+	/* Get the INPUT argument from `argp_parse', which we know is
+	   a pointer to our plugin list pointer. */
+	switch (key) {
+	case 128:
+	  debug = true;
+	  break;
+	case 'c':
+	  connect_to = arg;
+	  break;
+	case 'i':
+	  interface = arg;
+	  break;
+	case 'd':
+	  keydir = arg;
+	  break;
+	case 's':
+	  seckeyfile = arg;
+	  break;
+	case 'p':
+	  pubkeyfile = arg;
+	  break;
+	case 129:
+	  errno = 0;
+	  mc.dh_bits = (unsigned int) strtol(arg, NULL, 10);
+	  if (errno){
+	    perror("strtol");
+	    exit(EXIT_FAILURE);
+	  }
+	  break;
+	case 130:
+	  mc.priority = arg;
+	  break;
+	case ARGP_KEY_ARG:
+	  argp_usage (state);
+	  break;
+	  case ARGP_KEY_END:
+	    break;
+	default:
+	  return ARGP_ERR_UNKNOWN;
+	}
+	return 0;
+      }
+
+      struct argp argp = { .options = options, .parser = parse_opt,
+			   .args_doc = "",
+			   .doc = "Mandos client -- Get and decrypt"
+			   " passwords from mandos server" };
+      argp_parse (&argp, argc, argv, 0, 0, NULL);
+    }
+      
+    pubkeyfile = combinepath(keydir, pubkeyfile);
+    if (pubkeyfile == NULL){
+      perror("combinepath");
+      exitcode = EXIT_FAILURE;
+      goto end;
+    }
+    
+    seckeyfile = combinepath(keydir, seckeyfile);
+    if (seckeyfile == NULL){
+      perror("combinepath");
+      goto end;
+    }
+
+    ret = init_gnutls_global(&mc, pubkeyfile, seckeyfile);
+    if (ret == -1){
+      fprintf(stderr, "init_gnutls_global\n");
+      goto end;
+    } else {
+      gnutls_initalized = true;
+    }
+
+    uid = getuid();
+    gid = getgid();
+
+    ret = setuid(uid);
+    if (ret == -1){
+      perror("setuid");
+    }
+    
+    setgid(gid);
+    if (ret == -1){
+      perror("setgid");
+    }
+    
+    if_index = (AvahiIfIndex) if_nametoindex(interface);
+    if(if_index == 0){
+      fprintf(stderr, "No such interface: \"%s\"\n", interface);
+      exit(EXIT_FAILURE);
+    }
+    
+    if(connect_to != NULL){
+      /* Connect directly, do not use Zeroconf */
+      /* (Mainly meant for debugging) */
+      char *address = strrchr(connect_to, ':');
+      if(address == NULL){
+        fprintf(stderr, "No colon in address\n");
+	exitcode = EXIT_FAILURE;
+	goto end;
+      }
+      errno = 0;
+      uint16_t port = (uint16_t) strtol(address+1, NULL, 10);
+      if(errno){
+	perror("Bad port number");
+	exitcode = EXIT_FAILURE;
+	goto end;
+      }
+      *address = '\0';
+      address = connect_to;
+      ret = start_mandos_communication(address, port, if_index, &mc);
+      if(ret < 0){
+	exitcode = EXIT_FAILURE;
+      } else {
+	exitcode = EXIT_SUCCESS;
+      }
+      goto end;
+    }
+    
+    /* If the interface is down, bring it up */
+    {
+      sd = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
+      if(sd < 0) {
+	perror("socket");
+	exitcode = EXIT_FAILURE;
+	goto end;
+      }
+      strcpy(network.ifr_name, interface); /* Spurious warning */
+      ret = ioctl(sd, SIOCGIFFLAGS, &network);
+      if(ret == -1){
+	perror("ioctl SIOCGIFFLAGS");
+	exitcode = EXIT_FAILURE;
+	goto end;
+      }
+      if((network.ifr_flags & IFF_UP) == 0){
+	network.ifr_flags |= IFF_UP;
+	ret = ioctl(sd, SIOCSIFFLAGS, &network);
+	if(ret == -1){
+	  perror("ioctl SIOCSIFFLAGS");
+	  exitcode = EXIT_FAILURE;
+	  goto end;
+	}
+      }
+      close(sd);
+    }
+    
+    if (not debug){
+      avahi_set_log_function(empty_log);
+    }
+    
+    /* Initialize the pseudo-RNG for Avahi */
+    srand((unsigned int) time(NULL));
+    
+    /* Allocate main Avahi loop object */
+    mc.simple_poll = avahi_simple_poll_new();
+    if (mc.simple_poll == NULL) {
+        fprintf(stderr, "Avahi: Failed to create simple poll"
+		" object.\n");
+	exitcode = EXIT_FAILURE;
+        goto end;
+    }
+
+    {
+      AvahiServerConfig config;
+      /* Do not publish any local Zeroconf records */
+      avahi_server_config_init(&config);
+      config.publish_hinfo = 0;
+      config.publish_addresses = 0;
+      config.publish_workstation = 0;
+      config.publish_domain = 0;
+
+      /* Allocate a new server */
+      mc.server = avahi_server_new(avahi_simple_poll_get
+				   (mc.simple_poll), &config, NULL,
+				   NULL, &error);
+    
+      /* Free the Avahi configuration data */
+      avahi_server_config_free(&config);
+    }
+    
+    /* Check if creating the Avahi server object succeeded */
+    if (mc.server == NULL) {
+        fprintf(stderr, "Failed to create Avahi server: %s\n",
+		avahi_strerror(error));
+	exitcode = EXIT_FAILURE;
+        goto end;
+    }
+    
+    /* Create the Avahi service browser */
+    sb = avahi_s_service_browser_new(mc.server, if_index,
+				     AVAHI_PROTO_INET6,
+				     "_mandos._tcp", NULL, 0,
+				     browse_callback, &mc);
+    if (sb == NULL) {
+        fprintf(stderr, "Failed to create service browser: %s\n",
+		avahi_strerror(avahi_server_errno(mc.server)));
+	exitcode = EXIT_FAILURE;
+        goto end;
+    }
+    
+    /* Run the main loop */
+
+    if (debug){
+      fprintf(stderr, "Starting Avahi loop search\n");
+    }
+    
+    avahi_simple_poll_loop(mc.simple_poll);
+    
+ end:
+
+    if (debug){
+      fprintf(stderr, "%s exiting\n", argv[0]);
+    }
+    
+    /* Cleanup things */
+    if (sb != NULL)
+        avahi_s_service_browser_free(sb);
+    
+    if (mc.server != NULL)
+        avahi_server_free(mc.server);
+
+    if (mc.simple_poll != NULL)
+        avahi_simple_poll_free(mc.simple_poll);
+    free(pubkeyfile);
+    free(seckeyfile);
+
+    if (gnutls_initalized){
+      gnutls_certificate_free_credentials (mc.cred);
+      gnutls_global_deinit ();
+    }
+    
+    return exitcode;
+}

=== added file 'plugins.d/password-request.xml'
--- plugins.d/password-request.xml	1970-01-01 00:00:00 +0000
+++ plugins.d/password-request.xml	2008-08-08 01:31:58 +0000
@@ -0,0 +1,198 @@
+<?xml version='1.0' encoding='UTF-8'?>
+<?xml-stylesheet type="text/xsl"
+	href="http://docbook.sourceforge.net/release/xsl/current/manpages/docbook.xsl"?>
+<!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
+	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
+<!ENTITY VERSION "1.0">
+<!ENTITY COMMANDNAME "password-request">
+]>
+
+<refentry>
+  <refentryinfo>
+    <title>&COMMANDNAME;</title>
+    <!-- NWalsh's docbook scripts use this to generate the footer: -->
+    <productname>&COMMANDNAME;</productname>
+    <productnumber>&VERSION;</productnumber>
+    <authorgroup>
+      <author>
+	<firstname>Björn</firstname>
+	<surname>Påhlsson</surname>
+	<address>
+	  <email>belorn@fukt.bsnet.se</email>
+	</address>
+      </author>
+      <author>
+	<firstname>Teddy</firstname>
+	<surname>Hogeborn</surname>
+	<address>
+	  <email>teddy@fukt.bsnet.se</email>
+	</address>
+      </author>
+    </authorgroup>
+    <copyright>
+      <year>2008</year>
+      <holder>Teddy Hogeborn &amp; Björn Påhlsson</holder>
+    </copyright>
+    <legalnotice>
+      <para>
+	This manual page is free software: you can redistribute it
+	and/or modify it under the terms of the GNU General Public
+	License as published by the Free Software Foundation,
+	either version 3 of the License, or (at your option) any
+	later version.
+      </para>
+
+      <para>
+	This manual page is distributed in the hope that it will
+	be useful, but WITHOUT ANY WARRANTY; without even the
+	implied warranty of MERCHANTABILITY or FITNESS FOR A
+	PARTICULAR PURPOSE.  See the GNU General Public License
+	for more details.
+      </para>
+
+      <para>
+	You should have received a copy of the GNU General Public
+	License along with this program; If not, see
+	<ulink url="http://www.gnu.org/licenses/"/>.
+      </para>
+    </legalnotice>
+  </refentryinfo>
+
+  <refmeta>
+    <refentrytitle>&COMMANDNAME;</refentrytitle>
+    <manvolnum>8mandos</manvolnum>
+  </refmeta>
+  
+  <refnamediv>
+    <refname><command>&COMMANDNAME;</command></refname>
+    <refpurpose>
+      Client for mandos
+    </refpurpose>
+  </refnamediv>
+
+  <refsynopsisdiv>
+    <cmdsynopsis>
+      <command>&COMMANDNAME;</command>
+      <arg choice='opt' rep='repeat'>OPTION</arg>
+    </cmdsynopsis>
+  </refsynopsisdiv>
+
+  <refsect1 id="description">
+    <title>DESCRIPTION</title>
+    <para>
+      <command>&COMMANDNAME;</command> is a mandos plugin that works
+      like a client program that through avahi detects mandos servers,
+      sets up a gnutls connect and request a encrypted password. Any
+      passwords given is automaticly decrypted and passed to
+      cryptsetup.
+    </para>
+
+    <variablelist>
+      <varlistentry>
+	<term><literal>-c</literal>, <literal>--connect=<replaceable>
+	IP</replaceable></literal></term>
+	<listitem>
+	  <para>
+	    Connect directly to a sepcified mandos server
+	  </para>
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>-d</literal>, <literal>--keydir=<replaceable>
+	KEYDIR</replaceable></literal></term>
+	<listitem>
+	  <para>
+	    Directory where the openpgp keyring is
+	  </para>
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>-i</literal>, <literal>--interface=
+	<replaceable>INTERFACE</replaceable></literal></term>
+	<listitem>
+	  <para>
+	    Interface that Avahi will conntect through
+	  </para>
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>-p</literal>, <literal>--pubkey=<replaceable>
+	PUBKEY</replaceable></literal></term>
+	<listitem>
+	  <para>
+	    Public openpgp key for gnutls authentication
+	  </para>
+	</listitem>
+      </varlistentry>      
+
+      <varlistentry>
+	<term><literal>-s</literal>, <literal>--seckey=<replaceable>
+	SECKEY</replaceable></literal></term>
+	<listitem>
+	  <para>
+	    Secret openpgp key for gnutls authentication
+	  </para>
+	</listitem>
+      </varlistentry>            
+      
+      <varlistentry>
+	<term><literal>--priority=<replaceable>PRIORITY</replaceable>
+	</literal></term>
+	<listitem>
+	  <para>
+	    GNUTLS priority
+	  </para>
+	</listitem>
+      </varlistentry>            
+
+      <varlistentry>
+	<term><literal>--dh-bits=<replaceable>BITS</replaceable>
+	</literal></term>
+	<listitem>
+	  <para>
+	    dh-bits to use in gnutls communication
+	  </para>
+	</listitem>
+      </varlistentry>      
+      
+      <varlistentry>
+	<term><literal>--debug</literal></term>
+	<listitem>
+	  <para>
+	    Debug mode
+	  </para>
+	</listitem>
+      </varlistentry>
+      
+      <varlistentry>
+	<term><literal>-?</literal>, <literal>--help</literal></term>
+	<listitem>
+	  <para>
+	    Gives a help message
+	  </para>
+	</listitem>
+      </varlistentry>
+      
+      <varlistentry>
+	<term><literal>--usage</literal></term>
+	<listitem>
+	  <para>
+	    Gives a short usage message
+	  </para>
+	</listitem>
+      </varlistentry>
+
+      <varlistentry>
+	<term><literal>-V</literal>, <literal>--version</literal></term>
+	<listitem>
+	  <para>
+	    Prints the program version
+	  </para>
+	</listitem>
+      </varlistentry>            
+    </variablelist>
+  </refsect1>
+</refentry>

